package de.yourwebs.cent;

/**
 * @CentEngine.java     25.03.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 * Stellt die zentralen Methoden f&uuml;r den Zeichentausch bereit
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

import java.awt.Color;
import java.awt.Component;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import java.util.Hashtable;
import java.util.Map.Entry;
import java.util.regex.PatternSyntaxException;

import javax.swing.text.BadLocationException;
import javax.swing.text.Highlighter;
import javax.swing.text.JTextComponent;

public class CentEngine {

	private static final CentEngine centEngine = new CentEngine();

	/**
	 * die temp. Characters werden durch Klicken auf die Buttons des
	 * Sonderzeichen-Panels geladen Sie k&ouml;nnen (noch) nicht gespeichert
	 * werden.
	 */
	private String tempChars = "";

	/**
	 * die im Optionen-Dialog eingetragenen, in Entities zu tauschenden
	 * Characters
	 */
	private String[] tauschChar;

	private Properties properties = Config.getConfig().getProperties();

	private CentEngine() {
		createCharArray();
	}

	/**
	 * 
	 * @return die Singleton-Instanz dieser Klasse
	 */
	public static CentEngine getCentEngine() {
		return centEngine;
	}

	/**
	 * zentrale Methode des Zeichenaustauschs; ersetzt Sonderzeichen durch
	 * Entities. Im Optionen-Dialog kann zwischen HTML- und numerischen
	 * Unicode-Entities gewaehlt werden. Wenn HTML-Ersetzung gew&auml;hlt wurde
	 * wird bei Fehlen eines passenden Entities ein Unicode-Entity eingesetzt.
	 * 
	 * @param txt
	 *            String der zu bearbeitende Text
	 * @param html
	 *            boolean true wenn HTML-Entities gesetzt werden sollen
	 * @param allEntities
	 *            boolean true wenn alle Entities ersetzt werden sollen
	 * @param amp
	 *            boolean true wenn Ampersand auch durch Entity ersetzt werden
	 *            soll
	 * @return String der konvertierte Text
	 */
	public String tauschChar(String txt, boolean html, boolean allEntities,
			boolean amp) {

		// tauschChar neu einlesen
		createCharArray();
		// Rueckgabe des Originaltextes wenn keine Characters eingetragen sind
		if (tauschChar == null)
			return txt;

		// Lib aller Entities laden
		Hashtable<Character, String> ht = Config.getConfig().getLib();
		// keys in Enum einlesen
		Enumeration<Character> htEnum = ht.keys();
		boolean reverse = new Boolean(properties.getProperty("REVERSE"));
		boolean htmlMode = new Boolean(properties.getProperty("HTML"));

		// HTML-Mode konvertiert alles ohne '>' und '>' und bricht die Methode
		// danach ab
		if (htmlMode) {
			while (htEnum.hasMoreElements()) {
				Character c = htEnum.nextElement();
				// '&', '<' und '>' hier nicht konvertieren
				if (new Integer(c).toString().equals("60")
						|| new Integer(c).toString().equals("62")
						|| new Integer(c).toString().equals("38")
						|| new Integer(c).toString().equals("34")) {
					continue;
				}
				txt = replaceChar(txt, html, ht, c);
			}
			return txt;
		}

		// wenn 'reverse' eingestellt: alle Sonderzeichen AUSSER den
		// Eingetragenen konvertieren
		if (reverse) {
			int i = 0;
			while (htEnum.hasMoreElements()) {
				i++;
				Character c = htEnum.nextElement();
				// bei Vorhandensein im Eintrag: Austausch &uuml;berspringen
				if (Arrays.binarySearch(tauschChar, c.toString()) > -1) {
					continue;
				}
				// Ampersand hier nicht konvertieren wird ggfs. unten
				// konvertiert
				if (new Integer(c).toString().equals("38")) {
					continue;
				}
				txt = replaceChar(txt, html, ht, c);
			}
		}
		// wenn alle Sonderzeichen konvertiert werden sollen aber 'reverse'
		// nicht ausgewaehlt ist
		else if (allEntities && !reverse) {
			while (htEnum.hasMoreElements()) {
				Character c = htEnum.nextElement();
				// Ampersand hier nicht konvertieren wird ggfs. unten
				// konvertiert
				if (new Integer(c).toString().equals("38")) {
					continue;
				}
				txt = replaceChar(txt, html, ht, c);
			}
		} else {
			// wenn nur die eingetragenen Sonderzeichen konvertiert werden
			// sollen
			// Alle zu tauschenden Chars durchlaufen
			for (int i = 0; i < tauschChar.length; i++) {
				tauschChar[i] = tauschChar[i].trim();
				String zeichen = tauschChar[i];
				// numerische Eingabe konvertieren

				zeichen = strToChar(zeichen);
				// wenn HTML-Entities eingesetzt werden sollen und Entity in Lib
				// vorhanden
				if (html && ht.get(zeichen.charAt(0)) != null) {
					txt = txt.replaceAll(zeichen, ht.get(zeichen.charAt(0)));

					// wenn Unicode-Entities eingesetzt werden sollen
				} else {
					char c = zeichen.charAt(0);
					try {
						txt = txt.replaceAll("\\" + zeichen, "&#"
								+ new Integer(c).toString() + ";");
					} catch (PatternSyntaxException e) {
					}
				}
			}
		}

		// Ampersand wird nur konvertiert, wenn explizit gew&auml;hlt
		if (amp) {
			txt = replaceChar(txt, html, ht, '\u0026');
		}
		return txt;
	}

	/**
	 * zentrale Methode f&uuml;r das Ersetzen von einzelnen Chars durch
	 * Entities. In Abh&auml;ngigkeit von param html wird hier in HTML- oder
	 * Unicode-Entities konveriert
	 * 
	 * @param txt
	 *            String Text dessen Char getauscht werden sollen
	 * @param html
	 *            boolean true wenn HTML-Entities ausgegeben werden sollen,
	 *            sonst numerische Unicode-Entities
	 * @param ht
	 *            Hashtable der HTML-Entities
	 * @param c
	 *            Character der getauscht werden soll
	 * @return
	 */
	private String replaceChar(String txt, boolean html,
			Hashtable<Character, String> ht, Character c) {
		if (!html) {
			txt = txt.replace(c.toString(), "&#" + new Integer(c).toString()
					+ ";");
		} else {
			txt = txt.replace(c.toString(), ht.get(c));
		}
		return txt;
	}

	/**
	 * &Uuml;bergeordnete Methode der Ersetzen-Funktion. Es werden sowohl die im
	 * Optionen-Dialog, als auch die aktuell im Suchen-Dialog eingetragenen
	 * Strings ersetzt. Die im Optionen-Dialog fest eingesetzten Strings werden
	 * nur konvertiert wenn die Checkbox 'akiviert' angeklickt ist. Im
	 * Suchen-Dialog eingetragene Daten &uuml;berschreiben die im
	 * Optionen-Dialog fest eingetragenen.
	 * 
	 * @param txt
	 *            String, der zu bearbeitende Text
	 * @param orig
	 *            der auszutauschende String
	 * @param ersatz
	 *            der Ersatz-String
	 * @return der bearbeitete String
	 */
	public String replaceStr(String txt, String orig, String ersatz,
			boolean alle) {
		txt = txt.replaceAll(orig, ersatz);
		if (new Boolean(properties.getProperty("ACTIVATE_TABLE"))) {
			HashMap<Object, Object> hm = Config.getConfig().loadData();

			if (hm != null && alle) {
				Iterator<Entry<Object, Object>> it = hm.entrySet().iterator();
				while (it.hasNext()) {
					Map.Entry<Object, Object> entry = it.next();
					txt = ((String) txt).replaceAll((String) entry.getKey(),
							(String) entry.getValue());
				}
			}
		}
		return txt;
	}

	/**
	 * konvertiert eingetragene numerische Unicode-Werte-Strings zu Char-Strings
	 * Bsp. "37" -> "%"
	 * 
	 * @param zeichen
	 *            String eines numerischen Unicode-Wertes
	 * @return String eines Characters
	 */
	public String strToChar(String zeichen) {
		try {
			zeichen = new Character((char) new Integer(zeichen).intValue())
					.toString();
		} catch (NumberFormatException nfe) {

		}
		return zeichen;
	}

	/**
	 * Prozessiert den eingegebenen Characgter-String: Leerzeichen und
	 * abschliessende ',' werden entfernt
	 * 
	 * @param text
	 *            String der Zeichen
	 */
	public String processCharString(String text) {
		String s = text.trim();
		int i = s.length() - 1;
		// alle Kommata entfernen
		while (s.lastIndexOf(",") == i) {
			s = s.substring(0, i);
			i = s.length() - 1;
		}
		return s;
	}

	/**
	 * wechselt den Entity-Typ eines konvertierten Textes im Ausgabefeld
	 * 
	 * @param txt
	 *            String der konvertierte Text, in dem die Entities gewechselt
	 *            werden sollen. Bislang implementiert: Wechsel von HTML- zu
	 *            Unicode-Entities
	 * @param html
	 *            boolean, 'Richtung' des Tauschs: wenn true werden HTML-Entitys
	 *            zu Unicode-Entities getauscht, sonst umgekehrt
	 * @return der vollst&auml;ndig Text mit den konvertierten Entities
	 */
	public String changeEntity(String txt, boolean html) {
		// Lib der Entities
		Hashtable<Character, String> ht = Config.getConfig().getLib();
		Enumeration<Character> keys = ht.keys();
		// von HTML- zu Unicode-Entities konvertieren
		while (keys.hasMoreElements()) {
			Object c = keys.nextElement();
			String htmlEntity = new String((String) ht.get(c));
			String unicodeEntity = "&#"
					+ new Integer((int) ((Character) c).charValue()).toString()
					+ ";";
			if (html) {
				txt = txt.replaceAll(htmlEntity, unicodeEntity);
			} else {
				txt = txt.replaceAll(unicodeEntity, htmlEntity);
			}
		}
		return txt;
	}

	/**
	 * Dient dem Highlighting der Tausch-Chars
	 * 
	 * @param textComponent
	 *            JTextComponent die Komponente, deren Text behandelt werden
	 *            soll
	 * @param color
	 *            Color Farbe des Highlightings
	 * @return der Text der Komponente
	 */
	public void highlightChars(JTextComponent textComponent, Color color) {
		createCharArray();
		if (tauschChar == null) {
			return;
		}
		for (int i = 0; i < tauschChar.length; i++) {
			highlightString(textComponent, color, tauschChar[i]);
		}
	}

	/**
	 * Faerbt den Hintergrund unter einem String (Highlighting)
	 * 
	 * @param textComponent
	 *            Component die Komponente, deren Text behandelt werden soll
	 * @param color
	 *            Color Farbe des Highlightings
	 * @return der Text der Komponente
	 */
	public String highlightString(Component textComponent, Color color,
			String string) {
		String text = null;
		try {
			Highlighter hilite = ((JTextComponent) textComponent)
					.getHighlighter();
			text = ((JTextComponent) textComponent).getText();
			int pos = 0;
			Highlighter.HighlightPainter myHighlightPainter = new MyHighlightPainter(
					color);
			if (string.length() > 0) {
				while ((pos = text.indexOf(string, pos)) >= 0) {
					hilite.addHighlight(pos, pos + string.length(),
							myHighlightPainter);
					pos += string.length();
				}
			}
		} catch (BadLocationException e) {
		}
		return text;
	}

	/**
	 * addiert die &uuml;ber das Sonderzeichenpanel gew&auml;hlten
	 * tempor&auml;ren Sonderzeichen falls diese noch nicht eingetragen sind.
	 * 
	 * @param s
	 *            einzuf&uuml;gender String
	 */
	public void toggleTempChars(String s) {
		// Wenn Zeichen bereits vorhanden - entfernen
		if (tempChars.indexOf(s) != -1) {
			tempChars = tempChars.replaceAll("," + s, "");
			return;
		}
		Arrays.sort(tauschChar);
		if (Arrays.binarySearch(tauschChar, s) < 0) {
			tempChars += ("," + s);
		}

	}

	/**
	 * entfernt die &uuml;ber das Sonderzeichenpanel gew&auml;hlten
	 * tempor&auml;ren Sonderzeichen
	 */
	public void removeTempChars() {
		tempChars = "";
	}

	/**
	 * Liest die eingetragenen und temp. gew&auml;hlten Sonderzeichen in einen
	 * Komma-separierten String ein
	 * 
	 * @return
	 */
	public String getAllTauschCharString() {
		String chars = properties.getProperty("CHANGE_CHAR");
		chars += tempChars;
		return chars;
	}

	/**
	 * 
	 * @return einen Komma-separierten String aller auszutauschenden Strings der
	 *         'Ersetzen'-Tabelle
	 */
	public String getTauschStr() {
		HashMap<Object, Object> hm = Config.getConfig().loadData();
		String s = "";

		if (hm != null) {
			Iterator<Entry<Object, Object>> it = hm.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<Object, Object> entry = it.next();
				s = s.concat(entry.getKey() + ", ");
			}
		}
		s = s.trim();
		if (s.length() > 0)
			return s.substring(0, s.length() - 1);
		return s;
	}

	/**
	 * liest die zu konvertierenden Characters aus den Properties und laedt
	 * diese in die String[]-Instanzvariable 'tauschChar'
	 * 
	 * @return sortiertes String-Array der zu konvertierenden Zeichen falls leer
	 *         wird null zurueck gegeben
	 */
	public void createCharArray() {
		String chars = getAllTauschCharString();
		// in String[] zerlegen
		// wenn keine Tauschchars eingetragen Abbruch
		if (chars == null)
			return;
		// sonst: zu tauschende chars in Array laden und sortieren
		tauschChar = chars.split(",");
		Arrays.sort(tauschChar);
	}
}
