package de.yourwebs.cent;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;

import de.yourwebs.cent.components.MultiColorLabel;
import de.yourwebs.cent.components.CentToggleButton;
import de.yourwebs.cent.components.CentToolBar;
import de.yourwebs.cent.components.CharsetComboBox;
import de.yourwebs.cent.components.NumberedTextArea;
import de.yourwebs.cent.dialogs.AboutDialog;
import de.yourwebs.cent.dialogs.SearchDialog;
import de.yourwebs.cent.dialogs.options.OptionsDialog;

/**
 * @CentGUI.java     25.03.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * Stellt das GUI f&uuml;r den Sonderzeichenkonvertierer bereit. Die Klasse ist
 * als Singleton realisiert
 * 
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Cent dient zur Konvertierung von Sonderzeichen zu Entities. Hierzu
 * k&ouml;nnen Text-Dateien geladen und anschlei&szlig;end mit dem konvertierten
 * Text &uuml;berschrieben oder gesondert abgespeichert werden. Es kann zwischen
 * HTML- und numerischen Unicode-Entities gewechselt werden.|
 */
public class CentGUI extends JFrame implements ActionListener, ItemListener,
		CaretListener, ChangeListener {

	private static final long serialVersionUID = 1L;

	/**
	 * Startet die Konvertierung
	 */
	private JButton convertButt;

	/**
	 * leert die TextAreas
	 */
	private JButton clearButt;

	/**
	 * Toolbar-Buttons
	 */
	private JButton loadButt, saveButt, saveAsButt, saveOrigButt, changeButt,
			searchButt, removeNewEntitiesButt;

	/**
	 * Wechsel zwischen HTML- und Unicodeentities
	 */
	private CentToggleButton entityTypeButt;

	/**
	 * Ergebnis in Zwischenablage kopieren
	 */
	private CentToggleButton clipboardButt;

	/**
	 * Alle Sonderzeichen in Entities konvertieren
	 */
	private CentToggleButton allEntitiesButt;

	/**
	 * Alle Ampersand konvertieren
	 */
	private CentToggleButton ampButt;

	/**
	 * Zeilennummern
	 */
	private CentToggleButton lineNumbersButt;

	/**
	 * Zeilenumbruch bei Zeilennummern
	 */
	private CentToggleButton umbruchButt;

	/**
	 * Zeilenumbruch bei Textareas
	 */
	private CentToggleButton areaUmbruchButt;

	/**
	 * Auswahl des Charsets einer zu ladenden Datei
	 */
	private JComboBox<String> charSetBox;

	/**
	 * TextArea f&uuml;r den Originaltext
	 */
	private NumberedTextArea areaOrig;

	/**
	 * Tabulatorweite der TextAreas
	 */
	private int tabSize;

	/**
	 * TextArea f&uuml;r den konvertierten Text
	 */
	private NumberedTextArea areaConv;

	/**
	 * HTML-Darstellung des konvertierten Textes
	 */
	private JEditorPane areaHTML;

	private JTabbedPane tabbedPane;

	/**
	 * aktueller Dateipfad
	 */
	private String aktDatei;

	/**
	 * Labels in der Statusleiste
	 */
	private JLabel pfadLabel, posLabel, zeichenLabel;

	private MultiColorLabel modeLabel;

	/**
	 * Menus
	 */
	private JMenu dateiMenu, bearbeitenMenu, extrasMenu, hilfeMenu;

	/**
	 * MenuItems
	 */
	private JMenuItem openItem, saveAsItem, saveItem, exitItem, optionsItem,
			aboutItem, saveOrigItem, clearItem, convertItem, switchItem,
			helpItem, searchItem;

	/**
	 * aktuell fukussierte Komponente
	 */
	private Component aktComp;
	
	/**
	 * Pfad der aktuell ge&ouml;ffneten Datei
	 */
	private String lastOpened = "";

	/**
	 * Properties-Config
	 */
	private Properties properties = Config.getConfig().getProperties();

	/**
	 * Konvertierungsengine
	 */
	private CentEngine centEngine = CentEngine.getCentEngine();

	/**
	 * Zwischenablage
	 */
	CBoard clipboard = new CBoard();

	/**
	 * Singleton des GUI
	 */
	private static final CentGUI centGUI = new CentGUI();

	private CentGUI() {
		setSize();

		initComponents();

		this.setTitle(properties.getProperty("APP_NAME") + " "
				+ properties.getProperty("APP_VERSION"));
		this.setIconImage(new ImageIcon(this
				.getClass()
				.getClassLoader()
				.getResource(
						Config.IMG_VERZ + Config.APP_NAME.toLowerCase()
								+ "_logo.png")).getImage());
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setExtendedState(JFrame.MAXIMIZED_BOTH);
		this.setLocationRelativeTo(null);
		this.setTitle(Config.APP_NAME);
		this.setVisible(true);

		// nach Updates suchen
//		Updater centUpdater = new Updater();
//		centUpdater.checkUpdate();

	} // Ende Konstruktor

	/**
	 * initialisiert alle Komponenten
	 */
	private void initComponents() {
		// Separator-Dimension
		Dimension sepDim = new Dimension(20, 1);

		initMenubar();

		CentToolBar toolBar = new CentToolBar();

		initToggleButtons();

		initCharSetBox(toolBar);

		// Box der oberen Textarea und der linken oberen Buttons
		Box topBox = Box.createHorizontalBox();
		topBox.add(Box.createRigidArea(new Dimension(5, 5)));
		// Box der unteren Textarea
		Box bottomBox = Box.createHorizontalBox();
		bottomBox.add(Box.createRigidArea(new Dimension(5, 5)));
		// Box der Buttons im unteren Textarea-Bereich
		Box buttonBox = Box.createVerticalBox();
		buttonBox.setAlignmentY(TOP_ALIGNMENT);
		buttonBox.add(Box.createRigidArea(new Dimension(0, 3)));

		// Toolbarbuttons
//		loadButt = initButton(loadButt, null, "Datei laden",
//				"toolbarButtonGraphics.general", "Open24.gif", toolBar);
		loadButt = initButton(loadButt, null, "Datei laden",
                null, "open.png", toolBar);

		toolBar.addSeparator(sepDim);
		clearButt = initButton(clearButt, null, "Textbereiche leeren", null,
				"clear24.png", toolBar);
		searchButt = initButton(searchButt, null, "Suchen/Ersetzen", null,
				"search.png", toolBar);
		convertButt = initButton(convertButt, null, "Text konvertieren", null,
				"convert24.png", toolBar);
		changeButt = initButton(changeButt, null, "Entity-Typ tauschen", null,
				"change24.png", toolBar);
		toolBar.addSeparator(sepDim);
		toolBar.add(lineNumbersButt);
		toolBar.add(umbruchButt);
		toolBar.add(areaUmbruchButt);
		toolBar.addSeparator(sepDim);
		toolBar.add(entityTypeButt);
		toolBar.add(clipboardButt);
		toolBar.add(allEntitiesButt);
		toolBar.add(ampButt);

		// Buttons links oben
		saveOrigButt = initButton(saveOrigButt, null, "Original speichern",
                null, "save.png", topBox);
		saveOrigButt.setAlignmentY(Component.TOP_ALIGNMENT);
		// Abstand zwischen Buttons und Textfeld
		topBox.add(Box.createRigidArea(new Dimension(5, 0)));

		// Buttons links unten
		saveButt = initButton(saveButt, null, "Original \u00FCberschreiben",
				null, "save_as.png", buttonBox);
		saveButt.setAlignmentY(Component.TOP_ALIGNMENT);
		
		disableSaveButtons();

		saveAsButt = initButton(saveAsButt, null, "Speichern unter...",
				null, "save_as.png", buttonBox);
		saveAsButt.setAlignmentY(Component.TOP_ALIGNMENT);
		// Radiergummi-Button
		removeNewEntitiesButt = initButton(removeNewEntitiesButt, null,
				"Zeichenauswahl aufheben", null, "eraser.png", buttonBox);

		// TextAreas
		areaOrig = initAreas(areaOrig, true);
		areaConv = initAreas(areaConv, true);

		// Original-Bereich
		JScrollPane origPane = new JScrollPane(areaOrig,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		origPane.setAlignmentY(TOP_ALIGNMENT);
		topBox.add(origPane);

		// HTML-Bereich
		areaHTML = new JEditorPane();
		areaHTML.setContentType("text/html");
		JScrollPane htmlPane = new JScrollPane(areaHTML,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		// SonderzeichenBereich
		SonderzeichenPanel sonderPane = new SonderzeichenPanel(this);

		tabbedPane = new JTabbedPane();
		tabbedPane.addChangeListener(this);
		tabbedPane.setAlignmentY(TOP_ALIGNMENT);
		tabbedPane.add(areaConv, "Text");
		tabbedPane.add(topBox, "Text");
		tabbedPane.add(htmlPane, "HTML");
		tabbedPane.add(sonderPane, "Sonderzeichen");

		bottomBox.add(buttonBox);
		// Abstand zwischen Buttons und TabbedPane
		if (!Config.getOS().equals("mac")) {
			bottomBox.add(Box.createRigidArea(new Dimension(5, 0)));
		}
		bottomBox.add(tabbedPane);

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				topBox, bottomBox);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(Toolkit.getDefaultToolkit()
				.getScreenSize().height / 2);

		// Main-Layout
		setLayout(new BorderLayout());
		add(initStatusBar(), BorderLayout.SOUTH);
		add(splitPane, BorderLayout.CENTER);
		add(toolBar, BorderLayout.NORTH);
	} // initComponents

	private void setSize() {
		this.setSize(Config.APP_WIDTH, Config.APP_HEIGHT);
	}

	private void initCharSetBox(JComponent comp) {
		charSetBox = new CharsetComboBox();
		charSetBox.addItemListener(this);
		JLabel charLabel = new JLabel("Charset:");
		charLabel.setLabelFor(charSetBox);
		comp.add(charLabel);
		comp.add(charSetBox);
	}

	/**
	 * Factory-Methode zur Erzeugung der Buttons
	 * @param butt
	 *            JButton der Button
	 * @param txt
	 *            String Buttontext
	 * @param tooltip
	 *            String Tooltip
	 * @param pkg
	 *            String Resource-Package-Name der Icons
	 * @param iconName
	 *            String Iconname
	 * @param c
	 *            JComponent Komponente, dem der Button hinzugef&uuml;gt werden
	 *            soll
	 * @return JButton
	 * @throws IOException
	 */
	private JButton initButton(JButton butt, String txt, String tooltip,
			String pkg, String iconName, JComponent c) {
		Dimension iconDim = new Dimension(34, 34);
		butt = new JButton();
		ImageIcon icon = null;
		if (txt != null) {
			butt.setText(txt);
		}
		if (pkg != null) {
			try {
				icon = new ImageIcon(loadImageResource(pkg, iconName));
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			icon = new ImageIcon(getClass().getResource(
					"/" + Config.IMG_VERZ + iconName));
		}
		butt.setIcon(icon);
		butt.setPreferredSize(iconDim);
		butt.setSize(iconDim);
		if (tooltip != null) {
			butt.setToolTipText(tooltip);
		}
		butt.addActionListener(this);
		if (c != null)
			c.add(butt);
		return butt;
	}

	/**
	 * initialisiert die Toggle-Buttons
	 */
	private void initToggleButtons() {
		ClassLoader cl = this.getClass().getClassLoader();

		entityTypeButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "unicode24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ + "html24.png")));
		entityTypeButt.addActionListener(this);
		entityTypeButt.setToolTipText("Entity-Typ w\u00E4hlen");

		clipboardButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "clipboard24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "clipboard_full24.png")), new Boolean(
						properties.getProperty("CLIPBOARD")).booleanValue());
		clipboardButt.setToolTipText("In die Zwischenablage kopieren");

		allEntitiesButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "allEntities_not24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "allEntities24.png")));
		allEntitiesButt.setEnabled(!new Boolean(properties
				.getProperty("REVERSE")).booleanValue());
		allEntitiesButt.setToolTipText("Alle Sonderzeichen konvertieren");
		allEntitiesButt.addActionListener(null);
		allEntitiesButt.addItemListener(this);

		ampButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "ampersand_not24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "ampersand24.png")));
		ampButt.setEnabled(false);
		ampButt.setToolTipText("Ampersand konvertieren");

		lineNumbersButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "lineNumbers_not24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "lineNumbers24.png")));
		lineNumbersButt.addActionListener(this);
		lineNumbersButt.setToolTipText("Zeilennummerierung anzeigen");

		umbruchButt = new CentToggleButton(
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "umbruch_not24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ + "umbruch24.png")));
		umbruchButt.addActionListener(this);
		umbruchButt.setEnabled(false);
		umbruchButt
				.setToolTipText("Zeilenumbruch des Textes bei Zeilennummerierung ber\u00FCcksichtigen");

		areaUmbruchButt = new CentToggleButton(new ImageIcon(
				cl.getResource(Config.IMG_VERZ + "areaUmbruch_not24.png")),
				new ImageIcon(cl.getResource(Config.IMG_VERZ
						+ "areaUmbruch24.png")));
		areaUmbruchButt.addActionListener(this);
		areaUmbruchButt
				.setToolTipText("Zeilenumbruch des Textbereiches (de)aktivieren");
	}

	/**
	 * Factory-Methode zur Konfiguration der TextAreas
	 * 
	 * @param area
	 *            JTextArea
	 * @param wrap
	 *            boolean wenn true Zeilenumbruch aktiviert
	 */
	private NumberedTextArea initAreas(NumberedTextArea area, boolean wrap) {
		tabSize = new Integer(properties.getProperty("TABWIDTH")).intValue();
		area = new NumberedTextArea();
		area.setTabSize(tabSize);
		area.getTextArea().addCaretListener(this);
		return area;
	}

	/**
	 * Konfiguration der Menubar
	 */
	private void initMenubar() {
		JMenuBar menubar;
		menubar = new JMenuBar();

		dateiMenu = new JMenu("Datei");
		bearbeitenMenu = new JMenu("Bearbeiten");
		extrasMenu = new JMenu("Extras");
		hilfeMenu = new JMenu("Hilfe");

		menubar.add(dateiMenu);
		menubar.add(bearbeitenMenu);
		menubar.add(extrasMenu);
		menubar.add(hilfeMenu);

		initMenuItems();
		this.setJMenuBar(menubar);
	}

	/**
	 * Konfiguration der MenuItems
	 */
	private void initMenuItems() {
		openItem = new JMenuItem("\u00D6ffnen");
		openItem.addActionListener(this);
		openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				Event.ALT_MASK));
		saveOrigItem = new JMenuItem("Original speichern");
		saveOrigItem.addActionListener(this);
		saveOrigItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.ALT_MASK + Event.CTRL_MASK));
		saveItem = new JMenuItem("Original \u00FCberschreiben");
		saveItem.addActionListener(this);
		saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.ALT_MASK));
		saveAsItem = new JMenuItem("Speichern unter...");
		saveAsItem.addActionListener(this);
		saveAsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.ALT_MASK + Event.SHIFT_MASK));
		exitItem = new JMenuItem("Schlie\u00DFen");
		exitItem.addActionListener(this);
		exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
				Event.ALT_MASK));
		clearItem = new JMenuItem("Textbereiche l\u00F6schen");
		clearItem.addActionListener(this);
		clearItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
				Event.ALT_MASK));
		convertItem = new JMenuItem("Konvertieren");
		convertItem.addActionListener(this);
		switchItem = new JMenuItem("Entities tauschen");
		switchItem.addActionListener(this);
		searchItem = new JMenuItem("Suchen/Ersetzen");
		searchItem.addActionListener(this);
		optionsItem = new JMenuItem("Optionen");
		optionsItem.addActionListener(this);
		aboutItem = new JMenuItem("\u00DCber Cent");
		aboutItem.addActionListener(this);
		helpItem = new JMenuItem("Cent-Hilfe");
		helpItem.addActionListener(this);

		// Datei-Menu
		dateiMenu.add(openItem);
		dateiMenu.addSeparator();
		dateiMenu.add(saveItem);
		dateiMenu.add(saveOrigItem);
		dateiMenu.add(saveAsItem);
		dateiMenu.addSeparator();
		dateiMenu.add(exitItem);
		// Bearbeiten-Menu
		bearbeitenMenu.add(clearItem);
		bearbeitenMenu.add(convertItem);
		bearbeitenMenu.add(switchItem);
		bearbeitenMenu.add(searchItem);
		// Extras-Menu
		extrasMenu.add(optionsItem);
		// Hilfe-Menu
		hilfeMenu.add(aboutItem);
		hilfeMenu.add(helpItem);
	}

	/**
	 * Konfiguration der Statusleiste
	 * @return
	 */
	private JPanel initStatusBar() {
		EmptyBorder emptyBorder = new EmptyBorder(0, 8, 0, 8);
		pfadLabel = new JLabel("File: ");
		pfadLabel.setPreferredSize(new Dimension(400, 18));
		pfadLabel.setBorder(emptyBorder);

		posLabel = new JLabel(" ");
		posLabel.setPreferredSize(new Dimension(150, 18));
		posLabel.setBorder(emptyBorder);

		zeichenLabel = new JLabel(" ");
		zeichenLabel.setBorder(emptyBorder);

		Color[] colors = getModeColors();
		modeLabel = new MultiColorLabel(" ", colors[0], colors[1], colors[2]);
		modeLabel.setBorder(emptyBorder);

		configureStatusBar(centEngine.getAllTauschCharString());

		JPanel statusBar = new JPanel(new FlowLayout(FlowLayout.LEADING, 10, 0));
		statusBar.setPreferredSize(new Dimension(200, 18));
		statusBar.setSize(new Dimension(200, 18));
		statusBar.add(pfadLabel);
		statusBar.add(posLabel);
		statusBar.add(zeichenLabel);
		statusBar.add(Box.createHorizontalStrut(30));
		statusBar.add(modeLabel);
		return statusBar;
	}

	/**
	 * Setzt den Modus, die Modusfarben und die Konvertierungszeichen auf der
	 * Statusbar
	 * 
	 * @param text
	 *            String die Sonderzeichen
	 */
	public void configureStatusBar(String text) {
		setCharsToStatusBar(text);
		setModeToStatusBar();
		modeLabel.setColors(getModeColors());
	}
	
	/**
	 * setzt die eingetragenen Sonderzeichen auf die Statusleiste
	 * 
	 * @param text
	 *            String die Sonderzeichen
	 */
	private void setCharsToStatusBar(String text) {
		zeichenLabel.setOpaque(true);
		zeichenLabel.setText("Char: " + text);
	}

	/**
	 * Setzt den Modus-Text auf die Statusleiste
	 */
	private void setModeToStatusBar() {
		String mode = "Modus: ";
		if (properties.getProperty("HTML").equals("true")) {
			mode = mode + "HTML";
		} else {
			mode = mode + "Default";
		}
		modeLabel.setText(mode);
	}

	/**
	 * Erstellt das Array mit den Modus-Farbmarkierungen, die in der
	 * Statusleiste angezeigt werden. Die drei Farben werden einzeln dann
	 * angezeigt, wenn jeweils einer der Modi aktiviert ist. Die Liste zeigt die
	 * Farbmarkierungen von links nach rechts:
	 * <ul>
	 * <li>HTML</li>
	 * <li>orange</li>
	 * <li>Zeichen ausschlie&szlig;en</li>
	 * <li>blau</li>
	 * <li>Alle Entities</li>
	 * <li>rot</li>
	 * </ul>
	 * 
	 * @return das Array mir den Color-Objekten
	 */
	private Color[] getModeColors() {
		Color[] colors = new Color[] { null, null, null };
		if (properties.getProperty("HTML").equals("true"))
			colors[0] = Config.HTML_COLOR;
		else
			colors[0] = null;
		if (properties.getProperty("REVERSE").equals("true"))
			colors[1] = Config.REVERSE_COLOR;
		else
			colors[1] = null;
		if (allEntitiesButt.isSelected())
			colors[2] = Config.ALL_ENTITIES_COLOR;
		else
			colors[2] = null;
		return colors;
	}

	/**
	 * setzt die Daten der aktuellen Cursor-Position auf die Statusbar
	 * 
	 * @param a
	 *            JTextArea die aktuelle JTextArea
	 * @param cPos
	 *            int die Cursorposition
	 */
	private void setCaretPositionToStatusBar(int cPos) {
		try {
			// beginnt bei 0
			int lineNumber = ((JTextArea) aktComp).getLineOfOffset(cPos);

			int stOffset = ((JTextArea) aktComp).getLineStartOffset(lineNumber);
			String str = ((JTextArea) aktComp).getText(stOffset,
					(cPos - stOffset));
			int len = str.length();

			posLabel.setText("Zeile: " + new Integer(lineNumber + 1).toString()
					+ " | Spalte: " + new Integer(len + 1).toString());
		} catch (BadLocationException e1) {
		}
	}

	/**
	 * aufgerufen durch die Buttons des Sonderzeichen-Panels und den
	 * Radierbutton. L&auml;dt die Sonderzeichen-Anzeige in der Statusbar neu
	 * und erstellt ein neues Highlighting in der Textarea
	 */
	protected void reloadSonderzeichen() {
		// Highlighting anpassen
		centEngine.createCharArray();
		aktComp = (Component) ((NumberedTextArea) areaOrig).getTextArea();
		// setAktComp((Component) ((NumberedTextArea)
		// getOrigArea()).getTextArea());
		highlightChars();
		configureStatusBar(centEngine.getAllTauschCharString());
	}

	/**
	 * Initialisiert das Highlighting der Sonderzeichen auf den TextAreas
	 */
	private void highlightChars() {
		if (!(aktComp instanceof JTextArea))
			return;
		// Highlights entfernen
		MyHighlightPainter.removeHighlights((JTextArea) aktComp);
		if (new Boolean(properties.getProperty("HIGHLIGHT")).booleanValue()) {
			centEngine.highlightChars((JTextArea) aktComp, getHighlightColor());
			centEngine.highlightChars((JTextArea) areaOrig.getTextArea(),
					getHighlightColor());
		}
	}

	/**
	 * Erzeugt das Color-Objekt der Highlight-Farbe aus den gespeicherten
	 * Konfigurationsdateien
	 * 
	 * @return die Highlightfarbe
	 */
	private Color getHighlightColor() {
		int r = new Integer(properties.getProperty("HIGHLIGHT_RED")).intValue();
		int g = new Integer(properties.getProperty("HIGHLIGHT_GREEN"))
				.intValue();
		int b = new Integer(properties.getProperty("HIGHLIGHT_BLUE"))
				.intValue();
		return new Color(r, g, b);
	}

	/**
	 * setzt die Tabulatorweite der beiden TextAreas
	 */
	public void setTabSize() {
		tabSize = new Integer(properties.getProperty("TABWIDTH")).intValue();
		areaOrig.setTabSize(tabSize);
		areaConv.setTabSize(tabSize);
	}

	/**
	 * liefert den Text der Original-TextArea wenn Text markiert ist wird nur
	 * dieser zur&uuml;ck gegeben sonst der gesamte Text.
	 * 
	 * @return String Original-Text der linken TextArea
	 */
	public String getOrigAreaText() {
		if (areaOrig.getSelectedText() != null)
			return areaOrig.getSelectedText();
		return areaOrig.getText();
	}

	public JComboBox<String> getCharSetBox() {
		return charSetBox;
	}

	/**
	 * 
	 * @return String konvertierter Text des TextFeldes
	 */
	private String getConvAreaText() {
		return areaConv.getText();
	}

	public Component getAktComp() {
		if (aktComp != null)
			return aktComp;
		return areaOrig.getTextArea();
	}

	/**
	 * Setzt den konvertierten Text auf die unteren TextBereiche
	 * 
	 * @param text
	 *            String konvertierter Text
	 */
	private void setAreaBottomText(String text) {
		areaConv.setText(text);
		// filtert case-insensitiv <meta http-equiv...> aus
		// da JEditorPane HTML mit diesem Tag nicht darstellt
		text = text.replaceAll(
				"<((?i)(meta+[^>]*http-equiv))=[\"\'](.*?)[\"\']+.*?>", "");
		areaHTML.setText(text);
	}

	/**
	 * liest die aktuelle Datei aus
	 * 
	 * @return String der Text-Inhalt der aktuellen Datei
	 */
	private String readFile(String datei) {

		// Sicherheitsabfragen
		if (!verifyFile(datei)) {
			return null;
		}
		System.out.println("Lade Datei: " + datei);
		int c;
		StringBuffer buff = new StringBuffer();
		try {
			FileInputStream fis = new FileInputStream(datei);
			// wenn kein spezieller Charset gew&auml;hlt ist InputStreamReader
			// ohne initialisieren
			System.out.println("Verwendeter Zeichensatz: "
					+ properties.getProperty("CHARACTERSET"));
			InputStreamReader isr = new InputStreamReader(fis,
					properties.getProperty("CHARACTERSET"));
			BufferedReader rdr = new BufferedReader(isr);

			while ((c = rdr.read()) != -1) {
				buff.append((char) c);
			}
			rdr.close();

		} catch (IOException e) {
			JOptionPane.showMessageDialog(this,
					"IO-Fehler - Datei kann nicht gelesen werden!");
		}
		return buff.toString();
	}

	/**
	 * schreibt Text in eine Datei mit dem angegebenen Pfad
	 * 
	 * @param pfad
	 *            String der Pfad zur Datei
	 * @param txt
	 *            String zu schreibender Text
	 */
	private void writeFile(String pfad, String txt) {
		BufferedWriter buf = null;
		try {
			buf = new BufferedWriter(new FileWriter(pfad));
			buf.write(txt);
			buf.newLine();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (buf != null)
				try {
					buf.flush();
					buf.close();
				} catch (IOException e) {
				}
		}
	}

	/**
	 * l&auml;dt eine lokale Datei in die Original-Area
	 * 
	 * @param datName
	 *            String Pfad der Datei
	 */
	private void ladeDatei() {
		String txt = readFile(aktDatei);
		boolean alle = new Boolean(
				properties.getProperty("USE_TABLE_WHILE_STARTING"));
		if (alle) {
			int result = JOptionPane
					.showConfirmDialog(
							this,
							"Die Option zum automatischen Ersetzen einiger Zeichen ist aktiviert!\nSollen die eingetragenen Zeichen ersetzt werden?",
							"Zeichen automatisch ersetzen?",
							JOptionPane.OK_CANCEL_OPTION);
			if (result == JOptionPane.OK_OPTION)
				txt = centEngine.replaceStr(txt, "", "", alle);
		}
		areaOrig.setText(txt);
	}

	/**
	 * pr&uuml;ft den Dateipfad auf Existenz, Les- und Schreibbarkeit
	 * 
	 * @param datName
	 *            Pfad der zu pr&uuml;fenden Datei
	 * @return true wenn les- und schreibbar
	 */
	private boolean verifyFile(String datName) {
		// falls kein Pfad angegeben - Abbruch
		if (datName.equals("")) {
			JOptionPane.showMessageDialog(this, "Bitte eine Datei angeben!");
			return false;
		}
		// falls Datei nicht vorhanden - Abbruch
		File f = new File(datName);
		if (!f.canRead() || !f.isFile()) {
			System.err.println(f.toString()
					+ " nicht lesbar oder nicht vorhanden!");
			return false;
		}
		// falls Datei nicht beschreibbar - Abbruch
		if (!f.canWrite()) {
			System.err.println(f + " nicht beschreibbar!");
			return false;
		}
		return true;
	}

	/**
	 * 
	 * @return das GUI-Singleton
	 */
	public static CentGUI getCentGUI() {
		return centGUI;
	}

	/**
	 * deaktiviert den ToggleButton allEntitiesButt bei Auswahl von 'reverse'
	 * und aktiviert ihn bei Abwahl von 'reverse'
	 * 
	 * @param state
	 */
	public void enableAllEntitiesButtonState(boolean state) {
		allEntitiesButt.setEnabled(state);
	}

	private boolean oeffnen() {
		// oeffnen-Dialog zusamenbauen
		final JFileChooser chooser = new JFileChooser("Datei w\u00E4hlen");
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		// angezeigten Input-Pfad laden
		File file;
		if (properties.getProperty("LAST_OPENED") != null)
			file = new File(properties.getProperty("LAST_OPENED"));
		else
			file = new File(Config.LAST_OPENED);
		// wenn gueltiger Pfad, diesen laden, ansonsten wird home-Dir geladen
		if (file.isFile()) {
			chooser.setCurrentDirectory(file.getParentFile());
		}

		// Dialog sichtbar machen
		chooser.setVisible(true);
		// Oeffnen-Dialog anzeigen
		final int result = chooser.showOpenDialog(this);

		// wenn 'oeffnen' gewaehlt wurde
		if (result == JFileChooser.APPROVE_OPTION) {
			// Daten in die Textfelder setzen und Auslesen des
			// Eingangsverzeichnisses starten
			String s = chooser.getSelectedFile().toString();
			Config.getConfig().saveEntry("LAST_OPENED", s);
			lastOpened = s;
			aktDatei = s;
			ladeDatei();
			enableSaveButtons();
			return true;
		}
		// bei Abbruch
		chooser.setVisible(false);
		return false;
	}

	private boolean saveAs(String pfad) {

		JFileChooser chooser;
		if (pfad == null)
			pfad = System.getProperty("user.home");

		pfad = pfad.trim();
		File file = new File(pfad);

		chooser = new JFileChooser(pfad);
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		chooser.setDialogTitle("speichern unter...");
		chooser.setSelectedFile(file);

		// Filefilter
		chooser.addChoosableFileFilter(chooser.getAcceptAllFileFilter());
		FileFilter htmlFilter = new HTMLFilter();
		FileFilter textFilter = new TextFilter();
		FileFilter xmlFilter = new XMLFilter();
		chooser.addChoosableFileFilter(htmlFilter);
		chooser.addChoosableFileFilter(textFilter);
		chooser.addChoosableFileFilter(xmlFilter);

		// entsprechenden Filter setzen
		String endung = ".txt";
		chooser.setFileFilter(chooser.getAcceptAllFileFilter());
		if (!pfad.equals("")) {
			if (pfad.indexOf(".txt") != -1)
				chooser.setFileFilter(textFilter);
			if (pfad.toLowerCase().indexOf(".htm") != -1
					|| pfad.toLowerCase().indexOf(".html") != -1)
				chooser.setFileFilter(htmlFilter);
			if (pfad.toLowerCase().indexOf(".xml") != -1)
				chooser.setFileFilter(xmlFilter);
		}

		// Dialog sichtbar machen
		chooser.setVisible(true);

		// Speichern-Dialog anzeigen
		int result = chooser.showSaveDialog(this);

		// wenn 'speichern' nicht 'abbrechen' gew&auml;hlt wurde
		if (result == JFileChooser.APPROVE_OPTION) {

			String ff = chooser.getFileFilter().toString();

			// Filefilter auslesen und anzuh&auml;ngende Dateiendung danach
			// festlegen
			if (ff.indexOf("TextFilter") != -1) {
				endung = ".txt";
			} else if (ff.indexOf("HTMLFilter") != -1) {
				endung = ".htm";
			} else if (ff.indexOf("XMLFilter") != -1) {
				endung = ".xml";
			}

			pfad = chooser.getSelectedFile().toString();
			if (pfad.lastIndexOf(".") == -1)
				pfad = pfad + endung;

			save(pfad);

			Config.getConfig().saveEntry("LAST_SAVED", pfad);

			chooser.setVisible(false);
			return true;
		}
		chooser.setVisible(false);
		return false;
	}

	/**
	 * speichert den konvertierten Text in eine Datei
	 * Hier muss auch bei Auswahl der HTML-Registerkarte der Text aus der
	 * Text-Registerkarte gespeichert werden!
	 * 
	 * @param pfad
	 *            String Dateipfad
	 */
	private void save(String pfad) {
		if (tabbedPane.getSelectedIndex() == 0 || tabbedPane.getSelectedIndex() == 1) {
			save(pfad, getConvAreaText());
		}
	}

	/**
	 * Speichert einen Text <code>txt</code> unter dem Pfad <code>pfad</code>
	 * und legt ggf. eine Sicherungsdatei an.
	 * 
	 * @param pfad
	 * @param txt
	 */
	private void save(String pfad, String txt) {
		boolean sicher = new Boolean(properties.getProperty("SICHERUNG"))
				.booleanValue();
		if (sicher) {
			try {
				File file = new File(pfad + "~");
				String s = readFile(aktDatei);
				file.createNewFile();
				writeFile(file.getAbsolutePath(), s);
			} catch (IOException e1) {
				System.err
						.println("Fehler beim Schreiben der Sicherungskopie!");
				e1.printStackTrace();
			}
		}

		writeFile(pfad, txt);
	}
	
	/**
	 * aktiviert die beiden Speicherbuttons oben und unten
	 */
	private void enableSaveButtons(){
		saveButt.setEnabled(true);
		saveOrigButt.setEnabled(true);
	}
	
	/**
	 * deaktiviert die beiden Speicherbuttons oben und unten
	 */
	private void disableSaveButtons(){
		saveButt.setEnabled(false);
		saveOrigButt.setEnabled(false);
	}

	/**
	 * zeigt oder schließt die Zeilenanzeige der Textbereiche
	 * 
	 * @param set
	 */
	private void setLineNumbers(boolean set) {
		areaOrig.setLineNumbers(set);
		areaConv.setLineNumbers(set);
	}

	/**
	 * liest eine Datei aus einer *.jar
	 * 
	 * @param pkgname
	 * @param fname
	 * @return
	 */
	private InputStream getResourceStream(String pkgname, String fname) {
		String res = "/" + pkgname.replace('.', '/') + "/" + fname;
		Class<?> clazz = getClass();
		InputStream is = clazz.getResourceAsStream(res);
		return is;
	}

	/**
	 * Dient zum Laden der Icon-Dateien bei der Initialisierung der Buttons
	 * 
	 * @param pkgname
	 *            String, Paket, in dem die Image-Datei zu finden ist
	 * @param fname
	 *            String Dateiname der Image-Datei
	 * @return Image-Objekt des Icons
	 * @throws IOException
	 */
	private Image loadImageResource(String pkgname, String fname)
			throws IOException {
		Image ret = null;
		InputStream is = getResourceStream(pkgname, fname);
		if (is != null) {
			byte[] buffer = new byte[0];
			byte[] tmpbuf = new byte[1024];
			while (true) {
				int len = is.read(tmpbuf);
				if (len <= 0) {
					break;
				}
				byte[] newbuf = new byte[buffer.length + len];
				System.arraycopy(buffer, 0, newbuf, 0, buffer.length);
				System.arraycopy(tmpbuf, 0, newbuf, buffer.length, len);
				buffer = newbuf;
			}
			// create image
			ret = Toolkit.getDefaultToolkit().createImage(buffer);
			is.close();
		}
		return ret;
	}

	public void actionPerformed(ActionEvent e) {

		File lastOpendFile = new File(lastOpened);

		if (e.getSource() == convertButt || e.getSource() == convertItem) {
			setAreaBottomText(centEngine.tauschChar(getOrigAreaText(),
					entityTypeButt.isSelected(), allEntitiesButt.isSelected(),
					ampButt.isSelected()));
			if (clipboardButt.isSelected()) {
				clipboard.setClipboardContents(getConvAreaText());
			}
			areaConv.gibNummern(lineNumbersButt.isSelected());
		}
		if (e.getSource() == clearButt || e.getSource() == clearItem) {
			areaOrig.setText("");
			areaConv.setText("");
			areaHTML.setText("");
			areaOrig.gibNummern(lineNumbersButt.isSelected());
			areaConv.gibNummern(lineNumbersButt.isSelected());
			pfadLabel.setText("File: ");
			disableSaveButtons();
		}
		if (e.getSource() == loadButt || e.getSource() == openItem) {
			oeffnen();
			pfadLabel.setText("File: " + aktDatei);
		}
		if (e.getSource() == saveButt || e.getSource() == saveItem) {
			if (lastOpened.equals("")) {
				saveAs(lastOpened);
				return;
			}
			if (lastOpendFile.isFile() && lastOpendFile.canWrite()) {
				int result = JOptionPane
						.showConfirmDialog(
								this,
								"Soll die Datei "
										+ lastOpened
										+ "\nwirklich mit der konvertierten Version \u00FCberschrieben werden?");
				if (result == JOptionPane.OK_OPTION)
					save(lastOpened);
			} else {
				saveAs("");
			}
		}
		if (e.getSource() == saveOrigButt || e.getSource() == saveOrigItem) {
			if (lastOpendFile.isFile() && lastOpendFile.canWrite()) {
				int result = JOptionPane.showConfirmDialog(this,
						"Soll die Original-Datei " + lastOpened
								+ "\nwirklich \u00FCberschrieben werden?");
				if (result == JOptionPane.OK_OPTION)
					save(lastOpened, areaOrig.getText());
			} else {
				saveAs("");
			}
		}
		if (e.getSource() == saveAsButt || e.getSource() == saveAsItem) {
			saveAs(properties.getProperty("LAST_SAVED"));
		}
		if (e.getSource() == searchButt || e.getSource() == searchItem) {
			new SearchDialog();
		}
		if (e.getSource() == optionsItem) {
			new OptionsDialog();
		}
		if (e.getSource() == exitItem) {
			System.exit(0);
		}
		if (e.getSource() == aboutItem) {
			new AboutDialog(this);
		}
		if (e.getSource() == changeButt || e.getSource() == switchItem) {
			String txt = centEngine.changeEntity(areaConv.getText(),
					entityTypeButt.isSelected());
			entityTypeButt.setSelected(!entityTypeButt.isSelected());
			setAreaBottomText(txt);
		}
		if (e.getSource() == lineNumbersButt) {
			setLineNumbers(lineNumbersButt.isSelected());
			umbruchButt.setEnabled(lineNumbersButt.isSelected()
					&& areaUmbruchButt.isSelected());
		}
		if (e.getSource() == umbruchButt) {
			areaOrig.setNumberLineBreak(umbruchButt.isSelected());
			areaConv.setNumberLineBreak(umbruchButt.isSelected());
		}
		if (e.getSource() == areaUmbruchButt) {
			areaOrig.setWrap(areaUmbruchButt.isSelected());
			areaConv.setWrap(areaUmbruchButt.isSelected());
			umbruchButt.setEnabled(lineNumbersButt.isSelected()
					&& areaUmbruchButt.isSelected());
		}
		if (e.getSource() == helpItem) {
			File tmpIndexFile = null;
			InputStream is;
			StringBuffer sb = new StringBuffer();
			try {
				is = this.getClass().getClassLoader()
						.getResourceAsStream("help/index.htm");
				if (is != null) {
					int b = 0;
					while ((b = is.read()) != -1) {
						sb.append((char) b);
					}
					tmpIndexFile = File.createTempFile("Cent", ".htm");
					tmpIndexFile.deleteOnExit();
					if (tmpIndexFile.canWrite()) {
						PrintWriter pWriter = new PrintWriter(tmpIndexFile);
						pWriter.write(sb.toString());
						pWriter.close();
					} else {
						System.out.println("Tmp-File nicht beschreibbar");
						return;
					}
					Desktop.getDesktop().browse(tmpIndexFile.toURI());
				} else {
					System.out.println("Could not load resource.");
				}

			} catch (IOException e2) {
				e2.printStackTrace();
			}
		}
		if (e.getSource() == removeNewEntitiesButt) {
			centEngine.removeTempChars();
			reloadSonderzeichen();
		}
	}

	/**
	 * steuert die Einstellung von
	 * <ul>
	 * <li>Characterset-JComboBox</li>
	 * <li>Toggle-Button zum Wechsel zwischen den Modi zum Konvertieren aller
	 * oder der gew&auml;hlten Entities</li>
	 * </ul>
	 */
	public void itemStateChanged(ItemEvent e) {

		if (e.getSource() == charSetBox) {
			properties.setProperty("CHARACTERSET",
					(String) charSetBox.getSelectedItem());
			ladeDatei();
		}

		if (e.getSource() == allEntitiesButt) {
			if (!allEntitiesButt.isSelected())
				ampButt.setSelected(false);
			ampButt.setEnabled(allEntitiesButt.isSelected());
			modeLabel.setColors(getModeColors());
		}
	}

	public void caretUpdate(CaretEvent e) {
		JTextArea a = ((JTextArea) e.getSource());
		aktComp = a;
		int cPos = a.getCaretPosition();
		highlightChars();
		setCaretPositionToStatusBar(cPos);
	}

	/**
	 * Deaktiviert die Save-Buttons bei Auswahl der Sonderzeichen-Registerkarte
	 */
	public void stateChanged(ChangeEvent e) {
		//saveButt.setEnabled(true);
		saveAsButt.setEnabled(true);
		if (tabbedPane.getSelectedIndex() == 2) {
			saveButt.setEnabled(false);
			saveAsButt.setEnabled(false);
		}
	}
}

class CBoard implements ClipboardOwner {

	private Clipboard clipboard = Toolkit.getDefaultToolkit()
			.getSystemClipboard();

	public String getClipboardContents() {
		String result = "";

		Transferable contents = clipboard.getContents(null);
		boolean hasTransferableText = (contents != null)
				&& contents.isDataFlavorSupported(DataFlavor.stringFlavor);
		if (hasTransferableText) {
			try {
				result = (String) contents
						.getTransferData(DataFlavor.stringFlavor);
			} catch (UnsupportedFlavorException ex) {
				ex.printStackTrace();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		return result;
	}

	public void setClipboardContents(String s) {
		StringSelection stringSelection = new StringSelection(s);
		clipboard.setContents(stringSelection, this);
	}

	public void lostOwnership(Clipboard clipboard, Transferable contents) {
	}
}
