package de.yourwebs.cent;

import java.awt.Color;
import java.awt.Desktop;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.nio.charset.Charset;
import java.time.LocalDate;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Properties;

import de.yourwebs.cent.dialogs.AboutDialog;
import de.yourwebs.cent.dialogs.options.OptionsDialog;

/**
 * @Config.java     20.03.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Die Klasse definiert die Hashtable mit den Eintities, legt einige statische
 * Variablen fest und managed das Auslesen und Abspeichern der App-Properties
 * 
 */
public class Config {

	private static final Config config = new Config();

	private Properties properties = new Properties();

	public String configDir = System.getProperty("user.home")
			+ System.getProperty("file.separator") + ".cent";

	/**
	 * Pfad zur Konfigurationsdatei
	 */
	public String configFilePath;

	/**
	 * Pfad zur Serialisierungsdatei
	 */
	public String dataFilePath;

	/**
	 * Name der Konfigurationsdatei
	 */
	private static final String CONFIG_FILE_NAME = "Config.txt";

	/**
	 * Name der Serialisierungsdatei
	 */
	private static final String DATA_FILE_NAME = "Data.ser";

	/**
	 * Applicationname
	 */
	public static final String APP_NAME = "Cent";

	/**
	 * Application-Version
	 */
	public static final String APP_VERSION = "0.3.1";

	/**
	 * Build-Nr.
	 */
	public static final String BUILD = "231010-1";
	
	/**
	 * Copyright
	 */
	public static final String COPYRIGHT = "(c) javabeginners.de, 2008 - " + LocalDate.now().getYear();

	/**
	 * Downloadverzeichnis der App. wird bei Updates durchsucht
	 */
	public static final String DOWNLOAD_DIR = "http://www.javabeginners.de/Downloads/"
			+ APP_NAME + "/";

	/**
	 * URL des Downloads
	 */
	public static final String DOWNLOAD_URL = DOWNLOAD_DIR + getOS() + "/"
		+ APP_NAME + "_" + getOS() + ".zip";

	public static final String IMG_VERZ = "img/";

	/**
	 * Zeit, die der Splashscreen geoeffnet bleibt
	 */
	public static final int SPLASH_TIME = 3000;

	/**
	 * Defaultwert der Tabulatorweite der Textareas
	 */
	public static final String TABWIDTH = "4";

	/**
	 * Header der Konfigurationsdatei
	 */
	private static final String HEADER = "# Cent - Konfigurations-Datei\n# " + COPYRIGHT + "\n# \n# \n#";

	/**
	 * auml; ouml; uuml; Auml; Ouml; Uuml; szlig;
	 */
	public static final String DEFAULT_CHANGE_CHARS = "\u00E4,\u00F6,\u00FC,\u00C4,\u00D6,\u00DC,\u00DF";

	public static final String LAST_OPENED = System.getProperty("user.home");

	public static final int APP_WIDTH = Toolkit.getDefaultToolkit()
			.getScreenSize().width;

	public static final int APP_HEIGHT = Toolkit.getDefaultToolkit()
			.getScreenSize().height;

	public static final String HIGHLIGHT_RED = "220";
	public static final String HIGHLIGHT_BLUE = "220";
	public static final String HIGHLIGHT_GREEN = "220";
	public static final Color HIGHLIGHT_COLOR = new Color(220, 220, 220);

	public static final String SEARCH_RED = "255";
	public static final String SEARCH_BLUE = "240";
	public static final String SEARCH_GREEN = "240";
	public static final Color SEARCH_COLOR = new Color(255, 240, 240);

	public static final Color BORDER_COLOR = new Color(200, 200, 200);

	public static final Color ENTITY_COLOR = new Color(0, 0, 30);
	
	public static final Color HTML_COLOR = new Color(255, 179, 51);
	
	public static final Color REVERSE_COLOR = new Color(119, 183, 238);
	
	public static final Color ALL_ENTITIES_COLOR = new Color(255, 100, 100);

	/**
	 * 
	 */
	public static final String SONDERZEICHEN_STR = "<html>Zu konvertierende Sonderzeichen m\u00FCssen durch Kommata getrennt eingegeben werden!<br>ACHTUNG! Zu tauschende Zeichen, die keine HTML-Entity-Entsprechung besitzen, werden durch Unicode-Entities ersetzt.</html>";
	/**
	 * 
	 */
	public static final String SICHERUNG_STR = "<html>Legt eine Sicherungskopie mit der Endung '~' im Verzeichnis der Originaldatei an.</html>";
	/**
	 * 
	 */
	public static final String SPLASH_STR = "<html>Zeigt einen Splashscreen beim Programmstart an.</html>";
	/**
	 * 
	 */
	public static final String REVERSE_STR = "<html>Konvertiert ALLE Sonderzeichen AUSSER den angegebenen.</html>";
	/**
	 * 
	 */
	public static final String HTML_STR = "<html>Konvertiert alle Sonderzeichen mit Ausnahme von '&lt;', '&gt;', '&quot' und '&amp;'.</html>";
	/**
	 * 
	 */
	public static final String HIGHLIGHT_STR = "<html>Hervorhebung der zur Konvertierung eingetragenen Zeichen.</html>";

	/**
	 * Standardeinstellungen
	 */
	private Properties defaultProperties;

	private Config() {
		
		configFilePath = configDir + System.getProperty("file.separator")
				+ CONFIG_FILE_NAME;

		dataFilePath = configDir + System.getProperty("file.separator")
				+ DATA_FILE_NAME;
		initDefaultProperties();

		System.out.println("CONFIG. DIR.: " + configDir);
		System.out.println("CONFIG. FILE: " + configFilePath);
		System.out.println("DATA FILE: " + dataFilePath);

		// Properties auslesen und ggf. notwendige Dateien erzeugen
		loadProperties();
	}

	/**
	 * Default-Werte der Properties setzen
	 */
	private void initDefaultProperties() {
		defaultProperties = new Properties();
		defaultProperties.setProperty("APP_NAME", APP_NAME);
		defaultProperties.setProperty("APP_VERSION", APP_VERSION);
		defaultProperties.setProperty("LAST_OPENED", System
				.getProperty("user.home"));
		defaultProperties.setProperty("CONFIG_FILE_NAME", configFilePath);
		defaultProperties.setProperty("DATA_FILE_NAME", dataFilePath);
		defaultProperties.setProperty("LAST_FILE", "");
		defaultProperties.setProperty("SPLASH", "true");
		defaultProperties.setProperty("SICHERUNG", "false");
		defaultProperties.setProperty("CLIPBOARD", "true");
		defaultProperties.setProperty("REVERSE", "false");
		defaultProperties.setProperty("HTML", "false");
		defaultProperties.setProperty("HIGHLIGHT", "true");
		defaultProperties.setProperty("HIGHLIGHT_RED", HIGHLIGHT_RED);
		defaultProperties.setProperty("HIGHLIGHT_GREEN", HIGHLIGHT_GREEN);
		defaultProperties.setProperty("HIGHLIGHT_BLUE", HIGHLIGHT_BLUE);
		defaultProperties.setProperty("SEARCH_RED", SEARCH_RED);
		defaultProperties.setProperty("SEARCH_GREEN", SEARCH_GREEN);
		defaultProperties.setProperty("SEARCH_BLUE", SEARCH_BLUE);
		defaultProperties.setProperty("TABWIDTH", TABWIDTH);
		defaultProperties.setProperty("CHANGE_CHAR", DEFAULT_CHANGE_CHARS);
		defaultProperties.setProperty("ACTIVATE_TABLE", "true");
		defaultProperties.setProperty("USE_TABLE_WHILE_STARTING", "false");
		defaultProperties.setProperty("CHARACTERSET", Charset.defaultCharset()
				.toString());
	}

	/**
	 * 
	 * @return das (Singleton-) Config-Objekt
	 */
	public static Config getConfig() {
		return config;
	}

	/**
	 * 
	 * @return die Properties-Instanz
	 */
	public Properties getProperties() {
		return properties;
	}

	/**
	 * Erzeugt das Configurations-Verzeichnis und dessen Inhalt
	 * 
	 * @return true bei Erfolg Achtung! Auch wenn nur das
	 *         Konfigurationsverzeichnis nicht existiert und nicht angelegt
	 *         werden kann wird false zurueck gegeben
	 */
	private boolean makeConfigFiles() {

		try {
			File configFile = new File(configDir);
			if ((!configFile.isDirectory() && !configFile.mkdirs())
					|| !configFile.canWrite())
				return false;
			File cf = new File(configFilePath);
			File df = new File(dataFilePath);

			if (df.isFile() && cf.isFile()) {
				storeProperties(properties);
				saveEntry("CONFIG_FILE_NAME", configFilePath);
				saveEntry("DATA_FILE_NAME", dataFilePath);
				return true;
			}
			if (cf.createNewFile() && df.createNewFile()) {
				storeProperties(properties);
				saveEntry("CONFIG_FILE_NAME", configFilePath);
				saveEntry("DATA_FILE_NAME", dataFilePath);
				System.out.println(df + " und " + cf + " erstellt");
				return true;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * l&auml;dt die Properties-Datei wenn diese am Default-Ort gefunden wird
	 * und prueft ob sie lesbar ist und ueber die Dateigroesse, ob die Datei
	 * Eintraege enth&auml;lt. Die Qualitaet der Eintraege selbst wird nicht
	 * geprueft. Wenn die Datei nicht existiert, werden die Default-Properties
	 * geladen und versucht, die Datei zu erzeugen.
	 */
	private void loadProperties() {
		// falls vorhanden Properties auslesen
		File f = new File(configFilePath);
		Enumeration<?> propEnum = defaultProperties.propertyNames();
		String prop;

		if (f.canRead() && f.isFile() && f.length() > 0) {
			System.out.println(f + " ist lesbar");
			FileInputStream propIn = null;
			try {
				propIn = new FileInputStream(configFilePath);
				properties.load(propIn);
				// durchlaeuft die Keys der default Properties und prueft, ob
				// diese in
				// den ausgelesenen Properties belegt sind. Falls nicht wird ein
				// Leerstring gespeichert.
				while (propEnum.hasMoreElements()) {
					prop = (String) propEnum.nextElement();
					if (properties.getProperty(prop) == null) {
						properties.setProperty(prop, "");
					}
				}
				return;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					if (propIn != null)
						propIn.close();
				} catch (IOException e) {
				}
			}
		}
		// wenn Config-Datei nicht lesbar oder leer defaultProperties laden
		properties = defaultProperties;
		System.out
				.println(f
						+ " ist nicht lesbar oder nicht vorhaden, versuche sie zu erstellen...");
		makeConfigFiles();
	}

	/**
	 * schreibt die Properties-Datei neu
	 */
	private void storeProperties(Properties prop) {

		FileOutputStream propOut = null;
		try {
			propOut = new FileOutputStream(configFilePath);
			properties.putAll(prop);
			prop.store(propOut, HEADER + System.getProperty("line.separator"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (propOut != null)
					propOut.close();
			} catch (IOException e) {
			}
		}
	}

	/**
	 * Definiert die Hashtable aus der die Entities ausgelesen werden
	 * 
	 * @return Hashtable
	 */
	public Hashtable<Character, String> getLib() {
		// Archiv
		Hashtable<Character, String> ht = new Hashtable<Character, String>();
		ht.put('\u00E4', "&auml;");
		ht.put('\u00F6', "&ouml;");
		ht.put('\u00FC', "&uuml;");
		ht.put('\u00C4', "&Auml;");
		ht.put('\u00D6', "&Ouml;");
		ht.put('\u00DC', "&Uuml;");
		ht.put('\u00DF', "&szlig;");
		ht.put('\u00C2', "&Acirc;");
		ht.put('\u00E2', "&acirc;");
		ht.put('\u00B4', "&acute;");
		ht.put('\u00C6', "&AElig;");
		ht.put('\u00E6', "&aelig;");
		ht.put('\u00C0', "&Agrave;");
		ht.put('\u00E0', "&agrave;");
		ht.put('\u2135', "&alefsym;");
		ht.put('\u0391', "&Alpha;");
		ht.put('\u03B1', "&alpha;");
		ht.put('\u2227', "&and;");
		ht.put('\u2220', "&ang;");
		ht.put('\'', "&apos;");
		ht.put('\u00C5', "&Aring;");
		ht.put('\u00E5', "&aring;");
		ht.put('\u2248', "&asymp;");
		ht.put('\u00C3', "&Atilde;");
		ht.put('\u00E3', "&atilde;");
		ht.put('\u00C4', "&Auml;");
		ht.put('\u00E4', "&auml;");
		ht.put('\u201E', "&bdquo;");
		ht.put('\u0392', "&Beta;");
		ht.put('\u03B2', "&beta;");
		ht.put('\u00A6', "&brvbar;");
		ht.put('\u2022', "&bull;");
		ht.put('\u2229', "&cap;");
		ht.put('\u00C7', "&Ccedil;");
		ht.put('\u00E7', "&ccedil;");
		ht.put('\u00B8', "&cedil;");
		ht.put('\u00A2', "&cent;");
		ht.put('\u03A7', "&Chi;");
		ht.put('\u03C7', "&chi;");
		ht.put('\u02C6', "&circ;");
		ht.put('\u2663', "&clubs;");
		ht.put('\u2245', "&cong;");
		ht.put('\u00A9', "&copy;");
		ht.put('\u21B5', "&crarr;");
		ht.put('\u222A', "&cup;");
		ht.put('\u00A4', "&curren;");
		ht.put('\u2021', "&Dagger;");
		ht.put('\u2020', "&dagger;");
		ht.put('\u21D3', "&dArr;");
		ht.put('\u2193', "&darr;");
		ht.put('\u00B0', "&deg;");
		ht.put('\u0394', "&Delta;");
		ht.put('\u03B4', "&delta;");
		ht.put('\u2666', "&diams;");
		ht.put('\u00F7', "&divide;");
		ht.put('\u00C9', "&Eacute;");
		ht.put('\u00E9', "&eacute;");
		ht.put('\u00CA', "&Ecirc;");
		ht.put('\u00EA', "&ecirc;");
		ht.put('\u00C8', "&Egrave;");
		ht.put('\u00E8', "&egrave;");
		ht.put('\u2205', "&empty;");
		ht.put('\u2003', "&emsp;");
		ht.put('\u2002', "&ensp;");
		ht.put('\u0395', "&Epsilon;");
		ht.put('\u03B5', "&epsilon;");
		ht.put('\u2261', "&equiv;");
		ht.put('\u0397', "&Eta;");
		ht.put('\u03B7', "&eta;");
		ht.put('\u00D0', "&ETH;");
		ht.put('\u00F0', "&eth;");
		ht.put('\u00CB', "&Euml;");
		ht.put('\u00EB', "&euml;");
		ht.put('\u20AC', "&euro;");
		ht.put('\u2203', "&exist;");
		ht.put('\u0192', "&fnof;");
		ht.put('\u2200', "&forall;");
		ht.put('\u00BD', "&frac12;");
		ht.put('\u00BC', "&frac14;");
		ht.put('\u00BE', "&frac34;");
		ht.put('\u2044', "&frasl;");
		ht.put('\u0393', "&Gamma;");
		ht.put('\u03B3', "&gamma;");
		ht.put('\u2265', "&ge;");
		ht.put('\u003E', "&gt;");
		ht.put('\u21D4', "&hArr;");
		ht.put('\u2194', "&harr;");
		ht.put('\u2665', "&hearts;");
		ht.put('\u2026', "&hellip;");
		ht.put('\u00CD', "&Iacute;");
		ht.put('\u00ED', "&iacute;");
		ht.put('\u00CE', "&Icirc;");
		ht.put('\u00EE', "&icirc;");
		ht.put('\u00A1', "&iexcl;");
		ht.put('\u00CC', "&Igrave;");
		ht.put('\u00EC', "&igrave;");
		ht.put('\u2111', "&image;");
		ht.put('\u221E', "&infin;");
		ht.put('\u222B', "&int;");
		ht.put('\u0399', "&Iota;");
		ht.put('\u03B9', "&iota;");
		ht.put('\u00BF', "&iquest;");
		ht.put('\u2208', "&isin;");
		ht.put('\u00CF', "&Iuml;");
		ht.put('\u00EF', "&iuml;");
		ht.put('\u039A', "&Kappa;");
		ht.put('\u03BA', "&kappa;");
		ht.put('\u039B', "&Lambda;");
		ht.put('\u03BB', "&lambda;");
		ht.put('\u2329', "&lang;");
		ht.put('\u00AB', "&laquo;");
		ht.put('\u21D0', "&lArr;");
		ht.put('\u2190', "&larr;");
		ht.put('\u2308', "&lceil;");
		ht.put('\u201C', "&ldquo;");
		ht.put('\u2264', "&le;");
		ht.put('\u230A', "&lfloor;");
		ht.put('\u2217', "&lowast;");
		ht.put('\u25CA', "&loz;");
		ht.put('\u200E', "&lrm;");
		ht.put('\u2039', "&lsaquo;");
		ht.put('\u2018', "&lsquo;");
		ht.put('\u003C', "&lt;");
		ht.put('\u00AF', "&macr;");
		ht.put('\u2014', "&mdash;");
		ht.put('\u00B5', "&micro;");
		ht.put('\u00B7', "&middot;");
		ht.put('\u2212', "&minus;");
		ht.put('\u039C', "&Mu;");
		ht.put('\u03BC', "&mu;");
		ht.put('\u2207', "&nabla;");
		ht.put('\u00A0', "&nbsp;");
		ht.put('\u2013', "&ndash;");
		ht.put('\u2260', "&ne;");
		ht.put('\u220B', "&ni;");
		ht.put('\u00AC', "&not;");
		ht.put('\u2209', "&notin;");
		ht.put('\u2284', "&nsub;");
		ht.put('\u00D1', "&Ntilde;");
		ht.put('\u00F1', "&ntilde;");
		ht.put('\u039D', "&Nu;");
		ht.put('\u03BD', "&nu;");
		ht.put('\u00D3', "&Oacute;");
		ht.put('\u00F3', "&oacute;");
		ht.put('\u00D4', "&Ocirc;");
		ht.put('\u00F4', "&ocirc;");
		ht.put('\u0152', "&OElig;");
		ht.put('\u0153', "&oelig;");
		ht.put('\u00D2', "&Ograve;");
		ht.put('\u00F2', "&ograve;");
		ht.put('\u203E', "&oline;");
		ht.put('\u03A9', "&Omega;");
		ht.put('\u03C9', "&omega;");
		ht.put('\u039F', "&Omicron;");
		ht.put('\u03BF', "&omicron;");
		ht.put('\u2295', "&oplus;");
		ht.put('\u2228', "&or;");
		ht.put('\u00AA', "&ordf;");
		ht.put('\u00BA', "&ordm;");
		ht.put('\u00D8', "&Oslash;");
		ht.put('\u00F8', "&oslash");
		ht.put('\u00D5', "&Otild;");
		ht.put('\u00F5', "&otilde;");
		ht.put('\u2297', "&otimes;");
		ht.put('\u00D6', "&Ouml;");
		ht.put('\u00F6', "&ouml;");
		ht.put('\u00B6', "&para;");
		ht.put('\u2202', "&part;");
		ht.put('\u2030', "&permil;");
		ht.put('\u22A5', "&perp;");
		ht.put('\u03A6', "&Phi;");
		ht.put('\u03C6', "&phi;");
		ht.put('\u03A0', "&Pi;");
		ht.put('\u03C0', "&pi;");
		ht.put('\u03D6', "&piv;");
		ht.put('\u00B1', "&plusmn;");
		ht.put('\u00A3', "&pound;");
		ht.put('\u2033', "&Prime;");
		ht.put('\u2032', "&prime;");
		ht.put('\u220F', "&prod;");
		ht.put('\u221D', "&prop;");
		ht.put('\u03A8', "&Psi;");
		ht.put('\u03C8', "&psi;");
		ht.put('\u0022', "&quot;");
		ht.put('\u221A', "&radic;");
		ht.put('\u232A', "&rang;");
		ht.put('\u00BB', "&raquo;");
		ht.put('\u21D2', "&rArr;");
		ht.put('\u2192', "&rarr;");
		ht.put('\u2309', "&rceil;");
		ht.put('\u201D', "&rdquo;");
		ht.put('\u211C', "&real;");
		ht.put('\u00AE', "&reg;");
		ht.put('\u230B', "&rfloor;");
		ht.put('\u03A1', "&Rho;");
		ht.put('\u03C1', "&rho;");
		ht.put('\u200F', "&rlm;");
		ht.put('\u203A', "&rsaquo;");
		ht.put('\u2019', "&rsquo;");
		ht.put('\u201A', "&sbquo;");
		ht.put('\u0160', "&Scaron;");
		ht.put('\u0161', "&scaron;");
		ht.put('\u22C5', "&sdot;");
		ht.put('\u00A7', "&sect;");
		ht.put('\u00AD', "&shy;");
		ht.put('\u03A3', "&Sigma;");
		ht.put('\u03C3', "&sigma;");
		ht.put('\u03C2', "&sigmaf;");
		ht.put('\u223C', "&sim;");
		ht.put('\u2660', "&spades;");
		ht.put('\u2282', "&sub;");
		ht.put('\u2286', "&sube;");
		ht.put('\u2211', "&sum;");
		ht.put('\u2283', "&sup;");
		ht.put('\u00B9', "&sup1;");
		ht.put('\u00B2', "&sup2;");
		ht.put('\u00B3', "&sup3;");
		ht.put('\u2287', "&supe;");
		ht.put('\u00DF', "&szlig;");
		ht.put('\u03A4', "&Tau;");
		ht.put('\u03C4', "&tau;");
		ht.put('\u2234', "&there4;");
		ht.put('\u0398', "&Theta;");
		ht.put('\u03B8', "&theta;");
		ht.put('\u03D1', "&thetasym;");
		ht.put('\u2009', "&thinsp;");
		ht.put('\u00DE', "&THORN;");
		ht.put('\u00FE', "&thorn;");
		ht.put('\u02DC', "&tilde;");
		ht.put('\u00D7', "&times;");
		ht.put('\u2122', "&trade;");
		ht.put('\u00DA', "&Uacute;");
		ht.put('\u00FA', "&uacute;");
		ht.put('\u21D1', "&uArr;");
		ht.put('\u2191', "&uarr;");
		ht.put('\u00DB', "&Ucirc;");
		ht.put('\u00FB', "&ucirc;");
		ht.put('\u00D9', "&Ugrave;");
		ht.put('\u00F9', "&ugrave;");
		ht.put('\u00A8', "&uml;");
		ht.put('\u03D2', "&upsih;");
		ht.put('\u03A5', "&Upsilon;");
		ht.put('\u03C5', "&upsilon;");
		ht.put('\u00DC', "&Uuml;");
		ht.put('\u00FC', "&uuml;");
		ht.put('\u2118', "&weierp;");
		ht.put('\u039E', "&Xi;");
		ht.put('\u03BE', "&xi;");
		ht.put('\u00FD', "&yacute;");
		ht.put('\u00A5', "&yen;");
		ht.put('\u0178', "&Yuml;");
		ht.put('\u00FF', "&yuml;");
		ht.put('\u0396', "&Zeta;");
		ht.put('\u03B6', "&zeta;");
		ht.put('\u200D', "&zwj;");
		ht.put('\u200C', "&zwnj;");
		ht.put('\u0026', "&amp;");
		return ht;
	}

	/**
	 * f&uuml;gt ein key/value-Paar der Property-List hinzu, aktualisiert diese
	 * und schreibt sie in eine Datei CONFIG_FILE_NAME
	 * 
	 */
	public void saveEntry(String key, String value) {
		properties.setProperty(key, value);
		storeProperties(properties);
	}

	/**
	 * serialisiert die HashMap mit der Tabelle 'Ersetzen' des Optionen-Dialogs
	 * 
	 * @param dataFile
	 *            Speicher-Datei
	 * @param hm
	 *            zu serialisierende Hashmap
	 */
	public void saveData(HashMap<Object, Object> hm) {
		try {
			ObjectOutputStream aus = new ObjectOutputStream(
					new FileOutputStream(new File(dataFilePath)));
			aus.writeObject(hm);
			aus.flush();
			aus.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * laedt die serialisierte HashMap
	 * 
	 * @param dataFile
	 *            Speicher-Datei
	 * @return die HashMap
	 */
	@SuppressWarnings("unchecked")
	public HashMap<Object, Object> loadData() {
		HashMap<Object, Object> hm = null;
		ObjectInputStream in;
		try {
			File f = new File(dataFilePath);
			if (f.length() > 0) {
				in = new ObjectInputStream(new FileInputStream(f));
				hm = (HashMap<Object, Object>) in.readObject();
				in.close();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return hm;
	}

	/**
	 * gibt einen String-Identifier der aktuellen Plattform aus: Windows - 'win'
	 * MacOS - 'mac' alle anderen - 'noarch'
	 * 
	 * @return String-Identifier der aktuellen Plattform
	 */
	public static String getOS() {
		String osname = System.getProperty("os.name");
		if (osname != null && osname.toLowerCase().indexOf("mac") != -1) {
			return "mac";
		}
		if (osname != null && osname.toLowerCase().indexOf("windows") != -1) {
			return "win";
		}
		return "noarch";
	}
}
