/**
 * @ColoredButton.java     22.04.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 * Stellt einen einfachen Button mit farbigem Grund und ohne Hoover-Effekt bereit
 *
 * This file is a part of Javabeginners
 * Javabeginners is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Javabeginners is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Javabeginners;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;

import javax.swing.JButton;
import javax.swing.border.LineBorder;

import de.yourwebs.cent.Config;

public class ColoredButton extends JButton {

	private static final long serialVersionUID = 1L;
	private Color bgColor = Config.HIGHLIGHT_COLOR;
	private Color borderColor = Color.DARK_GRAY;
	private Color fgColor = Color.BLACK;
	private String title;
	private Rectangle buttRect = new Rectangle(0, 3, 80, 18);

	public ColoredButton() {
		this("");
	}
	
	public ColoredButton(int x, int y, int width, int height){
		this("");
		buttRect.x = x;
		buttRect.y = y;
		buttRect.width = width;
		buttRect.height = height;
	}

	public ColoredButton(String title) {
		super(title);
		this.title = title;
		this.setBorder(new LineBorder(borderColor, 1));
		this.setPreferredSize(new Dimension(80, 22));
	}
	
	/**
	 * 
	 * @param title String Titel
	 * @param bg Color Hintergrund
	 * @param fg Color Vordergrund
	 * @param bd Color Randfarbe
	 */
	public ColoredButton(String title, Color bg, Color fg, Color bd){
		this(title);
		this.bgColor = bg;
		this.fgColor = fg;
		this.borderColor = bd;
	}

	public void paint(Graphics g) {
		g.setColor(bgColor);
		g.fillRect(buttRect.x, buttRect.y, buttRect.width, buttRect.height);
		g.setColor(borderColor);
		g.drawRect(buttRect.x, buttRect.y, buttRect.width, buttRect.height - 2);
		g.setColor(fgColor);
		g.drawString(title, buttRect.x+5, buttRect.y+15);
	}

	/**
	 * setzt die Hintergrundfarbe
	 * 
	 * @param c
	 *            Color die Farbe
	 */
	public void setBackground(Color c) {
		bgColor = c;
		repaint();
	}
	
	public void setSize(Rectangle rect){
		buttRect = rect;
	}

	/**
	 * setzt die Vordergrundfarbe
	 * 
	 * @param c
	 *            Color die Farbe
	 */
	public void setForeground(Color c) {
		fgColor = c;
		repaint();
	}

	/**
	 * setzt die Randfarbe
	 * 
	 * @param c
	 *            Color die Farbe
	 */
	public void setBorderColor(Color c) {
		borderColor = c;
		repaint();
	}
}
