/**
 * @MultiColorLabel.java     05.01.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Javabeginners
 * Javabeginners is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Javabeginners is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Javabeginners;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.components;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;

import javax.swing.JLabel;

/**
 *
 * 
 */
public class MultiColorLabel extends JLabel {

	private Color leftColor, middleColor, rightColor;
	
	private String text;

	public MultiColorLabel(String txt, Color leftColor, Color middleColor, Color rightColor) {
		super(txt);
		this.text = txt;
		this.leftColor = leftColor;
		this.middleColor = middleColor;
		this.rightColor = rightColor;
		setOpaque(true);
		
	}

	public void paintComponent(Graphics g) {

		FontMetrics fm = this.getFontMetrics(getFont());
		this.setSize(fm.stringWidth(text) + 17*3, this.getHeight());
		int startX = fm.stringWidth(text) + 15;
		int markerWidth = 10;
		
		if(leftColor == null) leftColor = getBackground();
		if(middleColor == null) middleColor = getBackground();
		if(rightColor == null) rightColor = getBackground();
		
		g.setColor(leftColor);
		g.fillRect(startX, this.getHeight()/3, markerWidth, this.getHeight()/3);
		
		g.setColor(middleColor);
		g.fillRect(startX + markerWidth, this.getHeight()/3, markerWidth,
				this.getHeight()/3);
		
		g.setColor(rightColor);
		g.fillRect(startX + markerWidth*2, this.getHeight()/3, markerWidth,
				this.getHeight()/3);

		g.setColor(Color.GRAY);
		g.drawRect(startX, this.getHeight()/3, markerWidth,
				this.getHeight()/3);
		g.drawRect(startX + markerWidth, this.getHeight()/3, markerWidth,
				this.getHeight()/3);
		g.drawRect(startX + markerWidth*2, this.getHeight()/3, markerWidth,
				this.getHeight()/3);

		g.setColor(Color.DARK_GRAY);
		g.drawString(text, 0, this.getHeight()/2 + fm.getHeight()/2 - fm.getDescent());
	}

	public Color getLeftColor() {
		return leftColor;
	}

	public void setLeftColor(Color leftColor) {
		this.leftColor = leftColor;
		repaint();
	}

	public Color getMiddleColor() {
		return middleColor;
	}

	public void setMiddleColor(Color middleColor) {
		this.middleColor = middleColor;
	}

	public Color getRightColor() {
		return rightColor;
	}

	public void setRightColor(Color rightColor) {
		this.rightColor = rightColor;
		repaint();
	}
	
	public Color[] getColors(){
		return new Color[]{leftColor, middleColor, rightColor};
	}
	
	public void setColors(Color[] colors){
		leftColor = colors[0];
		middleColor = colors[1];
		rightColor = colors[2];
		repaint();
		validate();
	}
	
	public void setText(String txt){
		super.setText(txt);
		text = txt;
	}
}
