/**
 * @NumberedTextArea.java     04.08.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Javabeginners
 * Javabeginners is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Javabeginners is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Javabeginners;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.components;

import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;
import javax.swing.text.Utilities;

/**
 * realisiert eine JTextArea mit Zeilennummern, die sich auch beim
 * Ver&auml;ndern der Fenstergr&ouml;&szlig;e anpassen. Angezeigt werden die
 * Zeilennummern des in der JTextArea geladenen Textes, nicht die Zeilenzahl in
 * der JTextArea
 */
public class NumberedTextArea extends JScrollPane implements ComponentListener,
		KeyListener {

	private static final long serialVersionUID = 1L;

	private JTextArea textArea, numberArea;

	/**
	 * Zeilenumbruch bei Zeilennummern beachten
	 */
	private boolean numberLineBreak = false;
	
	/**
	 * Zeilenumbruch bei Textarea beachten
	 */
	private boolean areaLineBreak = false;
	
	

	public NumberedTextArea() {
		textArea = new JTextArea(20, 10);
		textArea.setMargin(new Insets(0, 10, 0, 10));
		textArea.addComponentListener(this);
		textArea.addKeyListener(this);
		// Textarea laden
		this.setViewportView(textArea);
		// horizontales Scrollen ist unn&uuml;tig, da die JTextArea den
		// Zeilenumbruch regelt
		this
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

		// eine JTextArea für die Zeilennummern erstellen und formatieren
		numberArea = new JTextArea();
		gibNummern(numberLineBreak);
		numberArea.setMargin(new Insets(0, 4, 0, 4));
		numberArea.setBackground(new Color(240, 240, 255));
		numberArea.setForeground(new Color(180, 180, 180));
		numberArea.setEditable(false);
		// einen header-Bereich des Scrollpane erzeugen und dort die
		// Zeilennummern darstellen
		setLineNumbers(numberLineBreak);
	}
	
	/**
	 * 
	 * @param set boolean wenn true wird der Bereich f&uuml;r die Zeilennummern angezeigt
	 */
	public void setLineNumbers(boolean set){
		if(set)
			this.setRowHeaderView(numberArea);
		else
			this.setRowHeaderView(null);
	}
	
	/**
	 * Zeilenumbruch der JTextArea
	 * @param wrap boolean wenn true werden Zeilen innerhalb der Textbereiche umgebrochen
	 */
	public void setWrap(boolean wrap){
		areaLineBreak = wrap;
		textArea.setLineWrap(wrap);
		textArea.setWrapStyleWord(wrap);
	}

	/**
	 * 
	 * @return die JTextArea-Komponente, &uuml;ber sie kann auf deren Methoden
	 *         zugegriffen werden
	 */
	public JTextArea getTextArea() {
		return textArea;
	}

	public void setText(String text) {
		textArea.setText(text);
	}
	
	public String getText(){
		return textArea.getText();
	}
	
	public String getSelectedText(){
		return textArea.getSelectedText();
	}

	public void setTabSize(int tabSize) {
		textArea.setTabSize(tabSize);
		textArea.repaint();
	}

	/**
	 * Managed die Darstellungsweise der Zeilennummen
	 * Liest die Anzahl der Zeilen aus der JTextArea aus und gibt deren
	 * fortlaufende Nummerierung durch '\n' umgebrochen als String zurS&uuml;ck
	 * 
	 * Unterschieden wird:
	 * Zeilenumbruch des geladenen Originaltextes innerhalb der JTextArea (areaLineBreak)
	 * Umbruch der Zeilenzahl: Unabh&auml;ngig von der Einstellung der JTextArea kann entschieden werden,
	 * ob die Darstellung der Zeilennummern 
	 *  - die Zeilen der JTextarea fortlaufend nummerieren soll oder ob
	 *  - nur der Zeilenanfang des geladenen Textes nummeriert werden soll.
	 *  
	 * @param umbruch boolean: true nummeriert nur die Anf&auml;nge der Zeilen des geladenen Textes
	 * 
	 * @return String die untereinander fortlaufende Zeilennummerierung
	 */
	public void gibNummern(boolean umbruch) {
		StringBuilder sb = new StringBuilder();
		// Anzahl aller Zeilen des Originaltextes ermitteln
		// Zeilenumbr&uuml;che der JTextArea werden nicht gez&auml;hlt
		int zeilenZahl = textArea.getLineCount();
		// Zeichenzahl des geladenen Textes ermitteln
		int textLength = textArea.getText().length();
		
		// wenn Zeilenumbruch der Textarea aktiviert ist
		if(!areaLineBreak & zeilenZahl != 0){
			for(int i=0; i<zeilenZahl; i++){
				sb.append(String.valueOf(i+1) + "\n");
			}
		}
		
		if (umbruch && areaLineBreak) {
			// Suchmuster des Zeilenumbruchs
			Pattern p = Pattern.compile(System
					.getProperty("line.separator"));
			// die erste Zeile nummerieren
			sb.append("1\n");
				// Z&auml;hlvariable die die Zeilenzahl angibt
				int n = 1;
				// einzelne Zeichen auslesen
				for (int i = 0; i < textLength; i++) {
					try {
						// wenn das ausgelesene Zeichen das letzte der Zeile
						// ist...
						int lineEnd = Utilities.getRowEnd(textArea, i);
						int lineStart = Utilities.getRowEnd(textArea, i + 1);
						if (lineEnd < lineStart) {
							// wenn Zeilennummerierung nach dem Originaltext
							// erfolgen soll
							if (numberLineBreak) {
								// ...diese Zeile von Beginn an auslesen
								String s = textArea.getText().substring(lineEnd, lineStart);
								// Die Zeile auf das Vorhandensein eines im
								// Originaltext enthaltenen
								// Zeilenumbruchs testen
								Matcher m = p.matcher(s);
								boolean result = m.find();
								if (!result) {
									// Wenn kein Zeilenumbruch vorhanden ist,
									// setzt
									// sich die Textzeile in der
									// n&auml;chsten TextArea-Zeile fort und es
									// darf
									// keine Zeilennummer
									// erscheinen, deshalb nur Textumbruch
									// erzeugen.
									sb.append("\n");
								} else {
									// wenn ein Zeilenumbruch vorhanden ist, den
									// Z&auml;hler hochsetzen
									// Zeilenzahl und einen folgenden
									// Zeilenumbruch
									// setzen
									n++;
									sb.append(String.valueOf(n) + "\n");
								}
								// wenn Zeilennummerierung nach der JTextArea
								// erfolgen soll
							} else {
								n++;
								sb.append(String.valueOf(n) + "\n");
							}
						}
					} catch (BadLocationException e1) {
						e1.printStackTrace();
					}
				}

		} else if(areaLineBreak) {
			int n = 0;
			
			for (int i = 0; i < textLength; i++) {
				try {
					if (Utilities.getRowEnd(textArea, i) < Utilities.getRowEnd(
							textArea, i + 1)) {
						n++;
						sb.append(String.valueOf(n) + "\n");
					}
				} catch (BadLocationException e1) {
					e1.printStackTrace();
				}
			}
			
			sb.append(String.valueOf(++n) + "\n");
		}
		// String der untereinander gesetzten Zeilennummern in Zeilennummernbereich laden
		numberArea.setText(sb.toString());
	}

	/**
	 * 
	 * @return boolean der Wert des lineBreak-Flags wenn true wird die
	 *         Zeilenzahl des Originaltextes angezeigt, sonst werden die Zeilen
	 *         der JTextArea fortlaufend nummeriert
	 */
	public boolean getLineBreak() {
		return numberLineBreak;
	}

	/**
	 * setzt das lineBreak-Flag für die Zeilennummern
	 * 
	 * @param lineBreak
	 *            boolean wenn true wird die Zeilenzahl des Originaltextes
	 *            angezeigt, sonst werden die Zeilen der JTextArea fortlaufend
	 *            nummeriert, default true
	 */
	public void setNumberLineBreak(boolean lineBreak) {
		this.numberLineBreak = lineBreak;
		gibNummern(lineBreak);
	}
	
	/**
	 * setzt das lineBreak-Flag f&uuml;r die Textarea
	 * 
	 * @param lineBreak
	 *            boolean wenn true wird die Zeilenzahl des Originaltextes
	 *            angezeigt, sonst werden die Zeilen der JTextArea fortlaufend
	 *            nummeriert, default true
	 */
	public void setAreaLineBreak(boolean lineBreak) {
		this.areaLineBreak = lineBreak;
	}

	// ComponentListener zur Reaktion auf Ver&auml;nderungen der Komponente
	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
	}

	public void componentResized(ComponentEvent e) {
		gibNummern(numberLineBreak);
	}

	public void componentShown(ComponentEvent e) {
	}

	// KeyListener zur Reaktionen auf Ver&auml;nderungen des Textes
	public void keyPressed(KeyEvent e) {
	}

	public void keyReleased(KeyEvent e) {
		gibNummern(numberLineBreak);
	}

	public void keyTyped(KeyEvent e) {
	}
}
