package de.yourwebs.cent.dialogs.options;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.nio.charset.Charset;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.yourwebs.cent.CentEngine;
import de.yourwebs.cent.CentGUI;
import de.yourwebs.cent.Config;

/**
 * @OptionsDialog.java 20.03.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla Stellt einen Optionen-Dialog bereit
 * 
 *         This file is a part of Cent Cent is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Cent is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Cent; if not, write to the Free Software Foundation, Inc.,
 *         59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
public class OptionsDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JButton okButt, acceptButt, exitButt;
	private OptionsPanel optionsPanel;
	private ErsetzenPanel ersetzenPanel;

	private CentGUI parent = CentGUI.getCentGUI();

	private Config config = Config.getConfig();

	private CentEngine centEngine = CentEngine.getCentEngine();

	public OptionsDialog() {
		super();
		optionsPanel = new OptionsPanel(this);
		ersetzenPanel = new ErsetzenPanel();
		initComponents();

		this.pack();
		this.setLocationRelativeTo(optionsPanel.getParent());
		this.setVisible(true);
	}

	private void initComponents() {

		// Buttons
		okButt = new JButton("ok");
		okButt.addActionListener(this);
		acceptButt = new JButton("\u00FCbernehmen");
		acceptButt.addActionListener(this);
		exitButt = new JButton("abbrechen");
		exitButt.addActionListener(this);
		JPanel buttPanel = new JPanel();
		buttPanel.add(okButt);
		buttPanel.add(acceptButt);
		buttPanel.add(exitButt);

		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(optionsPanel, "Allgemeines");
		tabbedPane.add(ersetzenPanel, "Ersetzen");
		this.add(tabbedPane, BorderLayout.CENTER);
		this.add(buttPanel, BorderLayout.SOUTH);
	}

	private void saveAll() {
		// String der eingetragenen Zeichen sichern
		config.saveEntry("CHANGE_CHAR", centEngine
				.processCharString(optionsPanel.getZeichenField().getText()));
		// Character-Set sichern
		config.saveEntry("CHARACTERSET", optionsPanel.getCharsetComboBox()
				.getSelectedItem().toString());
		// Tabweite sichern
		try {
			if (Integer.parseInt(optionsPanel.getTabField().getText()) > 0) {
				config.saveEntry("TABWIDTH", optionsPanel.getTabField()
						.getText());
				parent.setTabSize();
			}
		} catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this,
					"Bitte nur Zahlenwerte als Tabulatorweite eingeben!");
		}
		// Splashscreen-Anzeige sichern
		config.saveEntry("SPLASH", new Boolean(optionsPanel.getSplashCB()
				.isSelected()).toString());
		// Sicherungsdatei erstellen sichern
		config.saveEntry("SICHERUNG", new Boolean(optionsPanel.getSicherCB()
				.isSelected()).toString());
		// Reverse-Einstellung sichern
		config.saveEntry("REVERSE", new Boolean(optionsPanel.getReverseCB()
				.isSelected()).toString());
		// HTML-Einstellung sichern
		config.saveEntry("HTML", new Boolean(optionsPanel.getHtmlCB()
				.isSelected()).toString());
		// Highlighting sichern
		config.saveEntry("HIGHLIGHT", new Boolean(optionsPanel.getHighlightCB()
				.isSelected()).toString());

		// aktuelle Werte in GUI setzen
		// Character-Set-Einstellung aktualisieren
		String cSet = optionsPanel.getCharsetComboBox().getSelectedItem()
				.toString();
		parent.getCharSetBox().setSelectedItem(cSet);
		// Chars in Status-Leiste setzen
		parent.configureStatusBar(centEngine.getAllTauschCharString());

		// Status der Ersetzen-Tabelle speichern; vorher evtl. Editieren beenden
		ersetzenPanel.getErsetzenTabelle().stopCellEditing();
		ersetzenPanel.getErsetzenTabelle().getModel().saveModel();

		// Reverse-Einstellungen
		parent.enableAllEntitiesButtonState(!optionsPanel.getReverseCB()
				.isSelected());
	}

	/**
	 * TODO Abspeichern der Farbe anders loesen
	 * 
	 * @param type
	 * @param aktColor
	 */
	private void saveColor(String type, Color aktColor) {
		if (aktColor != null) {
			JButton aktButt = type.equals("HIGHLIGHT") ? optionsPanel
					.getHighlightColorButt() : optionsPanel
					.getSearchColorButt();
			aktButt.setBackground(aktColor);
			config.saveEntry(type + "_RED",
					new Integer(aktColor.getRed()).toString());
			config.saveEntry(type + "_GREEN",
					new Integer(aktColor.getGreen()).toString());
			config.saveEntry(type + "_BLUE",
					new Integer(aktColor.getBlue()).toString());
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == exitButt) {
			dispose();
		}
		if (e.getSource() == acceptButt) {
			saveAll();

		}
		if (e.getSource() == okButt) {
			saveAll();
			dispose();
		}
		if (e.getSource() == optionsPanel.getHighlightColorButt()) {
			Color aktColor = JColorChooser.showDialog(this,
					"Cent - Markierungsfarbe", Config.HIGHLIGHT_COLOR);
			saveColor("HIGHLIGHT", aktColor);
		}
		if (e.getSource() == optionsPanel.getSearchColorButt()) {
			Color aktColor = JColorChooser.showDialog(this,
					"Cent - Suchfarbe - ", Config.SEARCH_COLOR);
			saveColor("SEARCH", aktColor);
		}
		if (e.getSource() == optionsPanel.getSplashCB()) {
			optionsPanel.getHinweisLabel().setText("");
			if (optionsPanel.getSplashCB().isSelected())
				optionsPanel.getHinweisLabel().setText(Config.SPLASH_STR);
		}
		if (e.getSource() == optionsPanel.getSicherCB()) {
			optionsPanel.getHinweisLabel().setText("");
			if (optionsPanel.getSicherCB().isSelected())
				optionsPanel.getHinweisLabel().setText(Config.SICHERUNG_STR);
		}
		if (e.getSource() == optionsPanel.getReverseCB()) {
			optionsPanel.getHinweisLabel().setText("");
			if (optionsPanel.getReverseCB().isSelected())
				optionsPanel.getHinweisLabel().setText(Config.REVERSE_STR);
		}
		if (e.getSource() == optionsPanel.getHtmlCB()) {
			// HTML-Modus definiert einen Modus unter Ausschluss der
			// HTML-typischen Sonderzeichen
			optionsPanel.getReverseCB().setSelected(
					optionsPanel.getHtmlCB().isSelected());
			optionsPanel.getReverseCB().setEnabled(
					!optionsPanel.getHtmlCB().isSelected());
			optionsPanel.getZeichenField().setEnabled(
					!optionsPanel.getHtmlCB().isSelected());
			optionsPanel.getHinweisLabel().setText("");
			if (optionsPanel.getHtmlCB().isSelected()) {
				optionsPanel.getZeichenField().setText("\u003C,\u003E,\",&");
				optionsPanel.getHinweisLabel().setText(Config.HTML_STR);
			} else {
				optionsPanel.getZeichenField().setText(
						Config.DEFAULT_CHANGE_CHARS);
			}
		}
		if (e.getSource() == optionsPanel.getHighlightCB()) {
			optionsPanel.getHinweisLabel().setText("");
			if (optionsPanel.getHighlightCB().isSelected())
				optionsPanel.getHinweisLabel().setText(Config.HIGHLIGHT_STR);
		}
		if (e.getSource() == optionsPanel.getCharsetButt()) {
			optionsPanel.getCharsetComboBox().setSelectedItem(
					Charset.defaultCharset().displayName());
		}
	}
}
