/**
 * @OptionsPanel.java     07.09.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.dialogs.options;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import de.yourwebs.cent.Config;
import de.yourwebs.cent.components.CentPanel;
import de.yourwebs.cent.components.CharsetComboBox;
import de.yourwebs.cent.components.ColoredButton;

/**
 */
public class OptionsPanel extends CentPanel implements CaretListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Button f&uuml;r das Zur&uuml;cksetzen des CharSets auf Systemstandard
	 */
	private JButton charSetButt;
	/**
	 * Textfeld f&uuml;r den Eintrag der zu tauschenden Chars
	 */
	private JTextField zeichenField;
	/**
	 * Textfeld f&uuml;r den Eintrag der Tabulatorweite
	 */
	private JTextField tabField;
	/**
	 * 
	 */
	private JLabel zeichenLabel, splashLabel, hinweisLabel, sicherLabel,
			reverseLabel, htmlLabel, highlightLabel, charsetLabel, tabLabel,
			searchColorLabel;
	/**
	 * Checkbox f&uuml;r die Anzeige des Splashscreens
	 */
	private JCheckBox splashCB;
	/**
	 * Checkbox f&uuml;r die Aktivierung der Erstellung einer Sicherungskopie
	 */
	private JCheckBox sicherCB;
	/**
	 * Checkbox f&uuml;r die Reversschaltung diese bewirkt, dass alle
	 * Sonderzeichen bis auf die eingetragenen konvertiert werden
	 */
	private JCheckBox reverseCB;
	/**
	 * Checkbox f&uuml;r die Aktivierung des HTML-Modus. Er bewirkt, dass alle
	 * Sonderzeichen bis auf '<' und '>' konvertiert werden
	 */
	private JCheckBox htmlCB;
	/**
	 * Checkbox f&uuml;r die Aktivierung des Highlighting der zu konvertierenden
	 * Zeichen
	 */
	private JCheckBox highlightCB;
	/**
	 * Anzeige der Highlightfarbe, &Oouml;ffnet einen JColorDialog
	 */
	private ColoredButton highlightColorButt;
	/**
	 * Anzeige der Suchfarbe, &Oouml;ffnet einen JColorDialog
	 */
	private ColoredButton searchColorButt;
	/**
	 * Auswahl und Voreinstellung des CharSets
	 */
	private CharsetComboBox charsetComboBox;
	/**
	 * Referenz auf den Dialog; dient zur &Uuml;bergabe der
	 * AcrionListener-Referenzen
	 */
	private OptionsDialog dialog;
	/**
	 * gespeicherte Eigenschaften
	 */
	private Properties properties = Config.getConfig().getProperties();

	public OptionsPanel(OptionsDialog dialog) {
		this.dialog = dialog;
		initComponents();

		add(hinweisLabel, constraints, 0, 0, 4, 1, this);
		add(zeichenLabel, constraints, 0, 1, 1, 1, this);
		add(zeichenField, constraints, 1, 1, 3, 1, this);
		add(reverseLabel, constraints, 0, 2, 1, 1, this);
		add(reverseCB, constraints, 1, 2, 1, 1, this);
		add(htmlLabel, constraints, 0, 3, 1, 1, this);
		add(htmlCB, constraints, 1, 3, 1, 1, this);
		add(charsetLabel, constraints, 0, 4, 1, 1, this);
		add(charsetComboBox, constraints, 1, 4, 2, 1, this);
		add(charSetButt, constraints, 3, 4, 1, 1, this);
		add(splashLabel, constraints, 0, 5, 1, 1, this);
		add(splashCB, constraints, 1, 5, 1, 1, this);
		add(sicherLabel, constraints, 0, 6, 1, 1, this);
		add(sicherCB, constraints, 1, 6, 1, 1, this);
		add(highlightLabel, constraints, 0, 7, 1, 1, this);
		add(highlightCB, constraints, 1, 7, 1, 1, this);
		add(highlightColorButt, constraints, 2, 7, 1, 1, this);
		add(searchColorLabel, constraints, 0, 8, 1, 1, this);
		add(searchColorButt, constraints, 2, 8, 1, 1, this);
		add(tabLabel, constraints, 0, 9, 1, 1, this);
		add(tabField, constraints, 1, 9, 1, 1, this);
	} // Konstruktor

	private void initComponents() {

		hinweisLabel = new JLabel(Config.SONDERZEICHEN_STR);
		hinweisLabel.setForeground(Color.RED);
		hinweisLabel.setFont(new Font("SansSerif", Font.PLAIN, 11)
				.deriveFont(9F));
		hinweisLabel.setPreferredSize(new Dimension(400, 40));
		zeichenField = new JTextField(properties.getProperty("CHANGE_CHAR"), 15);
		zeichenField.addCaretListener(this);
		zeichenField.setEnabled(!new Boolean(properties
				.getProperty("HTML")).booleanValue());
		zeichenLabel = new JLabel("konvertieren von:");
		splashLabel = new JLabel("Splashscreen");
		splashCB = new JCheckBox("", new Boolean(properties
				.getProperty("SPLASH")).booleanValue());
		splashCB.addActionListener(dialog);
		sicherLabel = new JLabel("Sicherungsdatei");
		sicherCB = new JCheckBox("", new Boolean(properties
				.getProperty("SICHERUNG")).booleanValue());
		sicherCB.addActionListener(dialog);
		reverseLabel = new JLabel("Zeichen ausschlie\u00DFen");
		reverseCB = new JCheckBox("", new Boolean(properties
				.getProperty("REVERSE")).booleanValue());
		reverseCB.addActionListener(dialog);
		reverseCB.setEnabled(!new Boolean(properties
				.getProperty("HTML")).booleanValue());
		htmlLabel = new JLabel("HTML-Modus");
		htmlCB = new JCheckBox("", new Boolean(properties
				.getProperty("HTML")).booleanValue());
		htmlCB.addActionListener(dialog);
		highlightLabel = new JLabel("Highlighting");
		highlightCB = new JCheckBox("", new Boolean(properties
				.getProperty("HIGHLIGHT")).booleanValue());
		highlightCB.addActionListener(dialog);
		highlightColorButt = new ColoredButton(0, 3, 80, 18);
		highlightColorButt.addActionListener(dialog);
		highlightColorButt.setBackground(getButtonColor("HIGHLIGHT"));
		searchColorButt = new ColoredButton(0, 3, 80, 18);
		searchColorButt.addActionListener(dialog);
		searchColorButt.setBackground(getButtonColor("SEARCH"));
		charsetLabel = new JLabel("Character-Set");
		charsetComboBox = new CharsetComboBox();
		charSetButt = new JButton("System");
		charSetButt.addActionListener(dialog);
		tabLabel = new JLabel("Tabweite");
		tabField = new JTextField(properties.getProperty("TABWIDTH"), 2);
		searchColorLabel = new JLabel("Suche");
	}

	/**
	 * liest f&auml;r das Highlighting und die Suche die Farbwerte aus den
	 * Properties aus und gibt das entsprechende Color-Objekt zur&uuml;ck
	 * 
	 * @param String
	 *            der Farbtyp
	 * @return Color die Farbe f&uuml;r das Highlighting oder die Suche
	 */
	private Color getButtonColor(String color) {
		int r = 255, g = 255, b = 255;

		try {
			if (color.equals("HIGHLIGHT")) {
				r = new Integer(properties.getProperty("HIGHLIGHT_RED"));
				g = new Integer(properties.getProperty("HIGHLIGHT_GREEN"));
				b = new Integer(properties.getProperty("HIGHLIGHT_BLUE"));
			}
			if (color.equals("SEARCH")) {
				r = new Integer(properties.getProperty("SEARCH_RED"));
				g = new Integer(properties.getProperty("SEARCH_GREEN"));
				b = new Integer(properties.getProperty("SEARCH_BLUE"));
			}
		} catch (NumberFormatException nfe) {
		}
		return new Color(r, g, b);
	}

	/**
	 * @return the zeichenField
	 */
	public JTextField getZeichenField() {
		return zeichenField;
	}

	/**
	 * @return the tabField
	 */
	public JTextField getTabField() {
		return tabField;
	}

	/**
	 * @return the splashCB
	 */
	public JCheckBox getSplashCB() {
		return splashCB;
	}

	/**
	 * @return the sicherCB
	 */
	public JCheckBox getSicherCB() {
		return sicherCB;
	}

	/**
	 * @return the reverseCB
	 */
	public JCheckBox getReverseCB() {
		return reverseCB;
	}
	
	/**
	 * @return the htmlCB
	 */
	public JCheckBox getHtmlCB() {
		return htmlCB;
	}

	/**
	 * @return the highlightCB
	 */
	public JCheckBox getHighlightCB() {
		return highlightCB;
	}

	/**
	 * @return the highlightColorButt
	 */
	public ColoredButton getHighlightColorButt() {
		return highlightColorButt;
	}

	/**
	 * @return the searchColorButt
	 */
	public ColoredButton getSearchColorButt() {
		return searchColorButt;
	}

	/**
	 * @return the charsetComboBox
	 */
	public CharsetComboBox getCharsetComboBox() {
		return charsetComboBox;
	}

	public JButton getCharsetButt() {
		return charSetButt;
	}

	/**
	 * @return das Label f&uuml;r die Anzeige der Hinweise
	 */
	public JLabel getHinweisLabel() {
		return hinweisLabel;
	}

	public void caretUpdate(CaretEvent e) {
		hinweisLabel.setText(Config.SONDERZEICHEN_STR);
	}
}